#!/usr/bin/perl
#
#  '$RCSfile$'
#  Copyright: 2000 Regents of the University of California 
#
#   '$Author: sgarg $'
#     '$Date: 2004-02-05 16:36:26 -0800 (Thu, 05 Feb 2004) $'
# '$Revision: 2015 $' 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
# This is a CGI application for inserting metadata documents into
# the Metacat database.  It utilizes the Metacat.pm module for most work.
# In this script, we process the form fields passed in from a POST, insert a
# metadata document and an ACL document.

use Metacat;
use AppConfig qw(:expand :argcount);
use XML::LibXML;
use XML::LibXSLT;
use Template;
use Net::SMTP;
use CGI qw/:standard :html3/;
use strict;

# Global configuration paramters
#my $cfgdir = "@install-dir@";
#my $cfgdir = "/usr/local/devtools/tomcat/webapps/knb/style/skins";
my $cfgdir = "@install-dir@@style-skins-relpath@";
my $tmpdir = "@temp-dir@";
my $templatesdir = "@install-dir@@style-common-relpath@/templates";
my $now = time;
my $xslConvDir = "$cfgdir/lib/style/";

# Import all of the HTML form fields as variables
import_names('FORM');

# Set up the hash for returning data to the HTML templates
my $templateVars = { 'status' => 'success' };
my $error = 0;
my @errorMessages;

# create a new AppConfig object and load our config parameters
# note that this requires the form submission to have a "cfg" paramter
# to determine which config file to load
my $config = AppConfig->new({
    GLOBAL => { ARGCOUNT => ARGCOUNT_ONE, } });

$config->define("metacatUrl");
$config->define("username");
$config->define("password");
$config->define("ldapUrl");
$config->define("defaultScope");
$config->define("organization");
$config->define("orgabbrev");
$config->define("orgurl");
$config->define("accesspubid");
$config->define("accesssysid");
$config->define("datasetpubid");
$config->define("datasetsysid");
$config->define("mailhost");
$config->define("sender");
$config->define("recipient");
$config->define("adminname");
if ($FORM::cfg eq 'nceas') {
    $config->define("nceas_db");
    $config->define("nceas_db_user");
    $config->define("nceas_db_password");
}
$config->define("responseTemplate", { DEFAULT => 'crap.tmpl'} );
$config->define("entryFormTemplate", { DEFAULT => 'crap.tmpl'} );
$config->define("guideTemplate", { DEFAULT => 'crap.tmpl'} );
$config->define("confirmDataTemplate", { DEFAULT => 'crap.tmpl'} );
$config->define("deleteDataTemplate", { DEFAULT => 'crap.tmpl'} );
$config->define("debug", { DEFAULT => '0'} );
$config->define("lat", { ARGCOUNT => ARGCOUNT_HASH} );
$config->define("lon", { ARGCOUNT => ARGCOUNT_HASH} );

if (! hasContent($FORM::cfg)) {
    $error = "Application misconfigured.  Please contact the administrator.";
    push(@errorMessages, $error);
} else {
    my $cfgfile = $cfgdir . "/" . $FORM::cfg . "/" . $FORM::cfg . ".cfg";
    $config->file($cfgfile);
}

my $metacatUrl = $config->metacatUrl();
my $username = $config->username();
my $password = $config->password();
my $ldapUrl = $config->ldapUrl();
my $defaultScope = $config->defaultScope();
my $organization = $config->organization();
my $orgabbrev = $config->orgabbrev();
my $orgurl = $config->orgurl();
my $orgfilter = $organization;
   $orgfilter =~ s/ /%20/g;
my $responseTemplate = $config->responseTemplate();
my $entryFormTemplate = $config->entryFormTemplate();
my $deleteDataTemplate = $config->deleteDataTemplate();
my $guideTemplate = $config->guideTemplate();
my $confirmDataTemplate = $config->confirmDataTemplate();
my $accesspubid = $config->accesspubid();
my $accesssysid = $config->accesssysid();
my $datasetpubid = $config->datasetpubid();
my $datasetsysid = $config->datasetsysid();
my $mailhost = $config->mailhost();
my $sender = $config->sender();
my $recipient = $config->recipient();
my $adminname = $config->adminname();
my $nceas_db;
my $nceas_db_user;
my $nceas_db_password;
if ($FORM::cfg eq 'nceas') {
    $nceas_db = $config->nceas_db();
    $nceas_db_user = $config->nceas_db_user();
    $nceas_db_password = $config->nceas_db_password();
}
my $debug = $config->debug();
my $lat = $config->get('lat');
my $lon = $config->get('lon');

# Convert the lat and lon configs into usable data structures
my @sitelist;
my %siteLatDMS;
my %siteLongDMS;
foreach my $newsite (keys %$lat) {
    my ($latd, $latm, $lats, $latdir) = split(':', $lat->{$newsite});
    my ($lond, $lonm, $lons, $londir) = split(':', $lon->{$newsite});
    push(@sitelist, $newsite);
    $siteLatDMS{$newsite} = [ $latd, $latm, $lats, $latdir ];
    $siteLongDMS{$newsite} = [ $lond, $lonm, $lons, $londir ];
}

# set some configuration options for the template object
my $ttConfig = {
             INCLUDE_PATH => $templatesdir, 
             INTERPOLATE  => 0,                    
             POST_CHOMP   => 1,                   
             };

# create an instance of the template processor
my $template = Template->new($ttConfig) || die $Template::ERROR, "\n";

print "Content-type: text/html\n\n";

# Set up the template information that is common to all forms
$$templateVars{'cfg'} = $FORM::cfg;
$$templateVars{'recipient'} = $recipient;
$$templateVars{'adminname'} = $adminname;
$$templateVars{'organization'} = $organization;
$$templateVars{'orgabbrev'} = $orgabbrev;
$$templateVars{'orgurl'} = $orgurl;
$$templateVars{'orgfilter'} = $orgfilter;

debug("Registry: Initialized");
# Process the form based on stage parameter. 
if ($FORM::stage =~ "guide") {
    # Send back the information on how to fill the form
    $$templateVars{'section'} = "Guide on How to Complete Registry Entries";
    $template->process( $guideTemplate, $templateVars);
    exit(0);

} elsif ($FORM::stage =~ "insert") {
    # The user has entered the data. Do data validation and send back data 
    # to confirm the data that has been entered. 
    toConfirmData();
    exit(0);

}elsif ($FORM::dataWrong =~ "No, go back to editing" && $FORM::stage =~ "confirmed") {
    # The user wants to correct the data that he has entered. 
    # Hence show the data again in entryData form. 
    confirmDataToReEntryData();
    exit(0);

}elsif ($FORM::stage =~ "modify") {
    # Modification of a file has been requested. 
    # Show the form will all the values filled in.
    my @sortedSites;
    foreach my $site (sort @sitelist) {
        push(@sortedSites, $site);
    }
    $$templateVars{'siteList'} = \@sortedSites;
    $$templateVars{'section'} = "Modification Form";
    $$templateVars{'docid'} = $FORM::docid;
    modifyData();
    exit(0);

}elsif ($FORM::stage =~ "delete_confirm") {

    # Result from deleteData form. 
    if($FORM::deleteData =~ "Delete data"){
    # delete Data
    deleteData(1);    
    exit(0);
    } else {
    # go back to search page. 
    exit(0);
    }

}elsif ($FORM::stage =~ "delete") {
    # Deletion of a file has been requested. 
    # Ask for username and password using deleteDataForm
    $$templateVars{'docid'} = $FORM::docid;
    $template->process( $deleteDataTemplate, $templateVars);
    exit(0);

}elsif ($FORM::stage !~ "confirmed") {
    # None of the stages have been reached and data is not being confirmed. 
    # Hence, send back entry form for entry of data.  
    debug("Registry: Sending form");
    my @sortedSites;
    foreach my $site (sort @sitelist) {
        push(@sortedSites, $site);
    }
    
    if ($FORM::cfg eq 'nceas') {
        my $projects = getProjectList();
        $$templateVars{'projects'} = $projects;
        $$templateVars{'wg'} = \@FORM::wg;
    }

    $$templateVars{'siteList'} = \@sortedSites;
    $$templateVars{'section'} = "Entry Form";
    $$templateVars{'docid'} = "";
    debug("Registry: Sending form: ready to process template");
    $template->process( $entryFormTemplate, $templateVars);
    debug("Registry: Sending form: template processed");
    exit(0);
}

# Confirm stage has been reached. Enter the data into metacat. 

# Initialize some global vars
my $latDeg1 = "";
my $latMin1 = "";
my $latSec1 = "";
my $hemisphLat1 = "";
my $longDeg1 = "";
my $longMin1 = "";
my $longSec1 = "";
my $hemisphLong1 = "";
my $latDeg2 = "";
my $latMin2 = "";
my $latSec2 = "";
my $hemisphLat2 = "";
my $longDeg2 = "";
my $longMin2 = "";
my $longSec2 = "";
my $hemisphLong2 = "";

# validate the input form parameters
my $invalidParams;

if (! $error) {
    $invalidParams = validateParameters(1);
    if (scalar(@$invalidParams)) {
        $$templateVars{'status'} = 'failure';
        $$templateVars{'invalidParams'} = $invalidParams;
        $error = 1;
    }
}


my $metacat;
my $docid;
if (! $error) {
    # Parameters have been validated and Create the XML document

    my $xmldoc = createXMLDocument();

    # Write out the XML file for debugging purposes
    #my $testFile = $tmpdir . "/test.xml";

    # Create a  metacat object
    $metacat = Metacat->new();
    if ($metacat) {
        $metacat->set_options( metacatUrl => $metacatUrl );
    } else {
        #die "failed during metacat creation\n";
        push(@errorMessages, "Failed during metacat creation.");
    }

    # Login to metacat
    my $userDN = $FORM::username;
    my $userOrg = $FORM::organization;
    my $userPass = $FORM::password;
    my $dname = "uid=$userDN,o=$userOrg,dc=ecoinformatics,dc=org";
    
    my $errorMessage = "";
    my $response = $metacat->login($dname, $userPass);
    if (! $response) {
        push(@errorMessages, $metacat->getMessage());
        push(@errorMessages, "Failed during login.\n");
    }

    debug( "Registry: A");
    if ($FORM::docid eq "") {
        debug( "Registry: B1");
        # document is being inserted 
        my $notunique = "NOT_UNIQUE";
        while ($notunique eq "NOT_UNIQUE") {
            $docid = newAccessionNumber($defaultScope);
            
            $xmldoc =~ s/docid/$docid/;
    
            #my $testFile = "/tmp/test.xml";
            #open (TFILE,">$testFile") || die ("Cant open xml file...\n");
            #print TFILE $xmldoc;
            #close(TFILE);
    
            $notunique = insertMetadata($xmldoc, $docid);
            #  if (!$notunique) {
            # Write out the XML file for debugging purposes
            #my $testFile = $tmpdir . "/test-new.xml";
            #open (TFILE,">$testFile") || die ("Cant open xml file...\n");
            #print TFILE $newdoc;
            #close(TFILE);
            #   }

            # The id wasn't unique, so update our lastid file
            if ($notunique eq "NOT_UNIQUE") {
                debug( "Registry: Updating lastid (B1.1)");
                updateLastId($defaultScope);
            }
        }
        debug("Registry: B2");
        if ($notunique ne "SUCCESS") {
            debug("Registry: NO SUCCESS");
            debug("Message is: $notunique");
            push(@errorMessages, $notunique);
        }
        debug("Registry: B3");
    } else {
        # document is being modified
        $docid = $FORM::docid;
    
        my $x;
        my $y;
        my $z;

        ($x, $y, $z) = split(/\./, $docid); 
        $z++;
        $docid = "$x.$y.$z";
    
        $xmldoc =~ s/docid/$docid/;
        
        my $response = $metacat->update($docid, $xmldoc);

        if (! $response) {
            push(@errorMessages, $metacat->getMessage());
            push(@errorMessages, "Failed while updating.\n");  
        }

        if (scalar(@errorMessages)) {
            debug("Registry: ErrorMessages defined in modify.");
            $$templateVars{'status'} = 'failure';
            $$templateVars{'errorMessages'} = \@errorMessages;
            $error = 1;
        }

        #if (! $error) {
            #sendNotification($docid, $mailhost, $sender, $recipient);
        #}
    
        # Create our HTML response and send it back
        $$templateVars{'function'} = "modified";
        $$templateVars{'section'} = "Modification Status";
        $template->process( $responseTemplate, $templateVars);

        exit(0);
    }
}

debug("Registry: C");

if (scalar(@errorMessages)) {
    debug("Registry: ErrorMessages defined.");
    $$templateVars{'status'} = 'failure';
    $$templateVars{'errorMessages'} = \@errorMessages;
    $error = 1;
}

#if (! $error) {
#sendNotification($docid, $mailhost, $sender, $recipient);
#}

# Create our HTML response and send it back
$$templateVars{'function'} = "submitted";
$$templateVars{'section'} = "Submission Status";
$template->process( $responseTemplate, $templateVars);

exit(0);


################################################################################
#
# Subroutine for updating a metacat id for a given scope to the highest value
#
################################################################################
sub updateLastId {
  my $scope = shift;

  my $errormsg = 0;
  my $docid = $metacat->getLastId($scope);

  if ($docid =~ /null/) {
      # No docids with this scope present, so do nothing
  } elsif ($docid) {
      # Update the lastid file for this scope
      (my $foundScope, my $id, my $rev) = split(/\./, $docid);
      debug("Docid is: $docid\n");
      debug("Lastid is: $id");
      my $scopeFile = $cfgdir . "/" . $FORM::cfg . "/" . $scope . ".lastid";
      open(LASTID, ">$scopeFile") or 
          die "Failed to open lastid file for writing!";
      print LASTID $id, "\n";
      close(LASTID);
  } else {
    $errormsg = $metacat->getMessage();
    debug("Error in getLastId: $errormsg");
  }
}

################################################################################
#
# Subroutine for inserting a document to metacat
#
################################################################################
sub insertMetadata {
  my $xmldoc = shift;
  my $docid = shift;

  my $notunique = "SUCCESS";
  debug("Registry: Starting insert (D1)");
  my $response = $metacat->insert($docid, $xmldoc);
  if (! $response) {
    debug("Registry: Response gotten (D2)");
    my $errormsg = $metacat->getMessage();
    debug("Registry: Error is (D3): ".$errormsg);
    if ($errormsg =~ /is already in use/) {
      $notunique = "NOT_UNIQUE";
      #print "Accession number already used: $docid\n";
    } elsif ($errormsg =~ /<login>/) {
      $notunique = "SUCCESS";
    } else {
      #print "<p>Dumping error on failure...</p>\n";
      #print "<p>", $errormsg, "</p>\n";
      #die "Failed during insert\n";
      #print "<p>Failed during insert</p>\n";
      $notunique = $errormsg;
    }
  }
  debug("Registry: Ending insert (D4)");

  return $notunique;
}

################################################################################
#
# Subroutine for generating a new accession number
#  Note: this is not threadsafe, assumes only one running process at a time
#  Also: need to check metacat for max id # used in this scope already
################################################################################
sub newAccessionNumber {
  my $scope = shift;
    
  my $docrev = 1;
  my $lastid = 1;

  my $scopeFile = $cfgdir . "/" . $FORM::cfg . "/" . $scope . ".lastid";
  if (-e $scopeFile) {
    open(LASTID, "<$scopeFile") or die "Failed to generate accession number!";
    $lastid = <LASTID>;
    chomp($lastid);
    $lastid++;
    close(LASTID);
  }
  open(LASTID, ">$scopeFile") or die "Failed to open lastid file for writing!";
  print LASTID $lastid, "\n";
  close(LASTID);

  my $docroot = "$scope.$lastid.";
  my $docid = $docroot . $docrev;
  return $docid;
}

################################################################################
#
# Subroutine for generating a new ACL document
#
################################################################################
sub newAccessDocument {
  my $aclid = shift;
  my $acl = "<?xml version=\"1.0\"?>\n";
  $acl .= "<!DOCTYPE acl ";
  $acl .= "PUBLIC \"$accesspubid\" \"$accesssysid\">\n";
  $acl .= "<acl authSystem=\"ldap://ldap.ecoinformatics.org\" ";
  $acl .= "order=\"denyFirst\">\n";
  $acl .= "<identifier system=\"knb\">$aclid</identifier>\n";
  #$acl .= "<identifier>$aclid</identifier>\n";
  $acl .= "<allow><principal>$username</principal>" .
          "<permission>all</permission></allow>\n";
  #$acl .= "<allow><principal>public</principal>" .
  #        "<permission>read</permission></allow>\n";
  $acl .= "</acl>\n";

  return $acl;
}

################################################################################
#
# Subroutine for inserting identifers to the metadata document passed to us
#
################################################################################
sub insertIdentifiers {
  my $docstring = shift;
  my $aclid = shift;
  my $docid = shift;

  my $parser = XML::LibXML->new();
  my $dom = $parser->parse_string($docstring);

  my $root = $dom->documentElement;
  my $name = $root->getName();

  my $addedIdentifier = 0;
  my $currentElement = $root->getFirstChild();
  $name = $currentElement->getName();
  while ("$name" !~ "triple" &&
         "$name" !~ "temporalCov" && 
         "$name" !~ "geographicCov" && 
         "$name" !~ "taxonomicCov"
        ) {
    if ("$name" =~ "identifier" ||
        "$name" =~ "title") {
      if (! $addedIdentifier) {
      my $idelement = $dom->createElement( "identifier" );
        $addedIdentifier = 1;
        $idelement->setAttribute("system", "knb");
        $idelement->appendTextNode($docid);
        $root->insertBefore($idelement, $currentElement);
      }
    }
    $currentElement = $currentElement->getNextSibling();
    $name = $currentElement->getName();
  }
  # Link the document to the access doc
  my $element = $dom->createElement( "triple" );
  $element->appendTextChild( "subject", $aclid);
  $element->appendTextChild( "relationship", 
                             "describes access control rules for");
  $element->appendTextChild( "object", $docid);
  $root->insertBefore($element, $currentElement);
  # Link the access doc to the access doc
  $element = $dom->createElement( "triple" );
  $element->appendTextChild( "subject", $aclid);
  $element->appendTextChild( "relationship", 
                             "describes access control rules for");
  $element->appendTextChild( "object", $aclid);
  $root->insertBefore($element, $currentElement);

  return $dom->toString();
}

################################################################################
# 
# Validate the parameters to make sure that required params are provided
#
################################################################################
sub validateParameters {
    my $chkUser = shift;
    my @invalidParams;

    push(@invalidParams, "Provider's first name is missing.")
        unless hasContent($FORM::providerGivenName);
    push(@invalidParams, "Provider's last name is missing.")
        unless hasContent($FORM::providerSurName);
    push(@invalidParams, "Name of site is missing.")
        unless (hasContent($FORM::site) || $FORM::site =~ /elect/ ||
                $FORM::cfg eq "nceas");
    push(@invalidParams, "Data set title is missing.")
        unless hasContent($FORM::title);
    push(@invalidParams, "Originator's first name is missing.")
        unless hasContent($FORM::origNamefirst0);
    push(@invalidParams, "Originator's last name is missing.")
        unless hasContent($FORM::origNamelast0);
    push(@invalidParams, "Abstract is missing.")
        unless hasContent($FORM::abstract);
    push(@invalidParams, "Beginning year of data set is missing.")
        unless hasContent($FORM::beginningYear);
    push(@invalidParams, "Geographic description is missing.")
        unless (hasContent($FORM::geogdesc));

    # If the "use site" coord. box is checked and if the site is in 
    # the longitude hash ...  && ($siteLatDMS{$FORM::site})
   
    if (($FORM::useSiteCoord) && ($siteLatDMS{$FORM::site}) ) {
        
        $latDeg1 = $siteLatDMS{$FORM::site}[0];
        $latMin1 = $siteLatDMS{$FORM::site}[1];
        $latSec1 = $siteLatDMS{$FORM::site}[2];
        $hemisphLat1 = $siteLatDMS{$FORM::site}[3];
        $longDeg1 = $siteLongDMS{$FORM::site}[0];
        $longMin1 = $siteLongDMS{$FORM::site}[1];
        $longSec1 = $siteLongDMS{$FORM::site}[2];
        $hemisphLong1 = $siteLongDMS{$FORM::site}[3];
     
    }  else {
    
        $latDeg1 = $FORM::latDeg1;
        $latMin1 = $FORM::latMin1;
        $latSec1 = $FORM::latSec1;
        $hemisphLat1 = $FORM::hemisphLat1;
        $longDeg1 = $FORM::longDeg1;
        $longMin1 = $FORM::longMin1;
        $longSec1 = $FORM::longSec1;
        $hemisphLong1 = $FORM::hemisphLong1;
    }
    
    # Check if latDeg1 and longDeg1 has values if useSiteCoord is used. 
    # This check is required because some of the sites dont have lat 
    # and long mentioned in the config file. 

    if ($FORM::useSiteCoord) {
    push(@invalidParams, "The Data Registry doesn't have latitude and longitude information for the site that you chose. Please go back and enter the spatial information.")
        unless(hasContent($latDeg1) && hasContent($longDeg1));
    }else{
    push(@invalidParams, "Latitude degrees are missing.")
        unless hasContent($latDeg1);
    push(@invalidParams, "Longitude degrees are missing.")
        unless hasContent($longDeg1);
    }

    push(@invalidParams, 
        "You must provide a method description if you provide a method title.")
        if (hasContent($FORM::methodTitle) && ( !(scalar(@FORM::methodPara) > 0) 
                || (! hasContent($FORM::methodPara[0]))));
    push(@invalidParams, 
        "You must provide a method description if you provide a study extent description.")
        if (hasContent($FORM::studyExtentDescription) && (!(scalar(@FORM::methodPara) > 0) 
                || (! hasContent($FORM::methodPara[0]))));
    push(@invalidParams, 
        "You must provide both a study extent description and a sampling description, or neither.")
        if (
                (hasContent($FORM::studyExtentDescription) && !hasContent($FORM::samplingDescription)) ||
                (!hasContent($FORM::studyExtentDescription) && hasContent($FORM::samplingDescription))
           );

    push(@invalidParams, "Contact first name is missing.")
    unless (hasContent($FORM::origNamefirstContact) || 
        $FORM::useOrigAddress);
    push(@invalidParams, "Contact last name is missing.")
    unless (hasContent($FORM::origNamelastContact) || 
        $FORM::useOrigAddress);
    push(@invalidParams, "Data medium is missing.")
    unless (hasContent($FORM::dataMedium) || $FORM::dataMedium =~ /elect/);
    
    return \@invalidParams;
}

################################################################################
# 
# utility function to determine if a paramter is defined and not an empty string
#
################################################################################
sub hasContent {
    my $param = shift;

    my $paramHasContent;
    if (!defined($param) || $param eq '') { 
        $paramHasContent = 0;
    } else {
        $paramHasContent = 1;
    }
    return $paramHasContent;
}


################################################################################
# 
# Create the XML document from the HTML form input
# returns the XML document as a string
#
################################################################################
sub createXMLDocument {

    my $orig  = "";
    my $role  = "associatedParty";
    my $creat = "";
    my $metaP = "";
    my $apart = "";
    my $cont  = "";
    my $publ  = "";
    my $dso   = "";
    my $gmt = gmtime($now);
    my $doc =  "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";

   $doc .= "<eml:eml\n 
                     \t packageId=\"docid\" system=\"knb\"\n 
                     \t xmlns:eml=\"eml://ecoinformatics.org/eml-2.0.0\"\n
                     \t xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n 
                     \t xmlns:ds=\"eml://ecoinformatics.org/dataset-2.0.0\"\n 
                     \t xmlns:stmml=\"http://www.xml-cml.org/schema/stmml\"\n 
                     \t xsi:schemaLocation=\"eml://ecoinformatics.org/eml-2.0.0 eml.xsd\">\n";

    $doc .= "<!-- Person who filled in the catalog entry form: ";
    $doc .= "$FORM::providerGivenName $FORM::providerSurName -->\n";
    $doc .= "<!-- Form filled out at $gmt GMT -->\n";
    $doc .= "<dataset>\n";
    
    if (hasContent($FORM::identifier)) {
        $doc .= "<alternateIdentifier system=\"$FORM::site\">";
        $doc .= $FORM::identifier . "</alternateIdentifier>\n";
    }
    
    if (hasContent($FORM::title)) {
        $doc .= "<title>$FORM::title</title>\n";
    }

    if (hasContent($FORM::origNamelast0)) {
    $role = "creator";
        $orig .= "<individualName>\n";
        $orig .= "<givenName>$FORM::origNamefirst0</givenName>\n";
        $orig .= "<surName>$FORM::origNamelast0</surName>\n";
        $orig .= "</individualName>\n";
    }

    if (hasContent($FORM::origNameOrg)) {
        $orig .= "<organizationName>$FORM::origNameOrg</organizationName>\n";
    }

    if (hasContent($FORM::origDelivery) || hasContent($FORM::origCity) ||
        (hasContent($FORM::origState   ) &&
        ($FORM::origState !~ "Select state here.")) ||
        hasContent($FORM::origStateOther) ||
        hasContent($FORM::origZIP ) || hasContent($FORM::origCountry)) {
        $orig .= "<address>\n";

        if (hasContent($FORM::origDelivery)) {
            $orig .= "<deliveryPoint>$FORM::origDelivery</deliveryPoint>\n";
        }
        if (hasContent($FORM::origCity)) {
            $orig .= "<city>$FORM::origCity</city>\n";
        }

    if (hasContent($FORM::origState) && 
            ($FORM::origState !~ "Select state here.")) {
            $orig .= "<administrativeArea>$FORM::origState";
            $orig .= "</administrativeArea>\n";
        } elsif (hasContent($FORM::origStateOther)) {
            $orig .= "<administrativeArea>$FORM::origStateOther";
            $orig .= "</administrativeArea>\n";
        }
        if (hasContent($FORM::origZIP)) {
            $orig .= "<postalCode>$FORM::origZIP</postalCode>\n";
        }
        if (hasContent($FORM::origCountry)) {
            $orig .= "<country>$FORM::origCountry</country>\n";
        }
        $orig .= "</address>\n";
    }

    if (hasContent($FORM::origPhone)) {
        $orig .= "<phone>$FORM::origPhone</phone>\n";
    }
    if (hasContent($FORM::origFAX)) {
        $orig .= "<phone phonetype=\"Fax\">$FORM::origFAX</phone>\n";
    }
    if (hasContent($FORM::origEmail)) {
        $orig .= "<electronicMailAddress>$FORM::origEmail";
        $orig .= "</electronicMailAddress>\n";
    }
    $dso = "<$role>\n$orig</$role>\n";
    
    $creat .= $dso;

    if ($FORM::useOrigAddress) {
        # Add a contact originator like the original with a different role
            $cont .= "<contact>\n";
        $cont .= $orig;
        $cont .= "</contact>\n";
    } else {
        $cont .= "<contact>\n";

        $cont .= "<individualName>\n";
        $cont .= "<givenName>$FORM::origNamefirstContact</givenName>\n";
        $cont .= "<surName>$FORM::origNamelastContact</surName>\n";
        $cont .= "</individualName>\n";
 
    if (hasContent($FORM::origNameOrgContact)) {
        $cont .= "<organizationName>$FORM::origNameOrgContact</organizationName>\n";
    }

        if (hasContent($FORM::origDeliveryContact) || 
            hasContent($FORM::origCityContact) ||
            (hasContent($FORM::origStateContact) &&
            ($FORM::origStateContact !~ "Select state here.")) ||
            hasContent($FORM::origStateOtherContact) ||
            hasContent($FORM::origZIPContact) || 
            hasContent($FORM::origCountryContact)) {
            $cont .= "<address>\n";
            if (hasContent($FORM::origDeliveryContact)) {
                $cont .= "<deliveryPoint>$FORM::origDeliveryContact";
                $cont .= "</deliveryPoint>\n";
            }
            if (hasContent($FORM::origCityContact)) {
                $cont .= "<city>$FORM::origCityContact</city>\n";
            }
            if (hasContent($FORM::origStateContact) && 
                ($FORM::origStateContact !~ "Select state here.")) {
                $cont .= "<administrativeArea>$FORM::origStateContact";
                $cont .= "</administrativeArea>\n";
            } elsif (hasContent($FORM::origStateOtherContact)) {
                $cont .= "<administrativeArea>$FORM::origStateOtherContact";
                $cont .= "</administrativeArea>\n";
            }
            if (hasContent($FORM::origZIPContact)) {
                $cont .= "<postalCode>$FORM::origZIPContact</postalCode>\n";
            }
            if (hasContent($FORM::origCountryContact)) {
                $cont .= "<country>$FORM::origCountryContact</country>\n";
            }
            $cont .= "</address>\n";
        }
        if (hasContent($FORM::origPhoneContact)) {
            $cont .= "<phone>$FORM::origPhoneContact</phone>\n";
        }
    if (hasContent($FORM::origFAXContact)) {
        $cont .= "<phone phonetype=\"Fax\">$FORM::origFAXContact</phone>\n";
    }
        if (hasContent($FORM::origEmailContact)) {
            $cont .= "<electronicMailAddress>$FORM::origEmailContact";
            $cont .= "</electronicMailAddress>\n";
        }
    $cont .= "</contact>\n";
    }

    $metaP .= "<metadataProvider>\n";
    $metaP .= "<individualName>\n";
    $metaP .= "<givenName>$FORM::providerGivenName</givenName>\n";
    $metaP .= "<surName>$FORM::providerSurName</surName>\n";
    $metaP .= "</individualName>\n";
    $metaP .= "</metadataProvider>\n";

    # Additional originators
    foreach my $tmp (param()) {
        if ($tmp =~ /origNamelast/){
            my $tmp1 = $tmp;
            $tmp1 =~ s/origNamelast//; # get the index of the parameter 0 to 10
            if ( $tmp1 eq '1' 
                 || $tmp1 eq '2'
                 || $tmp1 eq '3'
                 || $tmp1 eq '4'
                 || $tmp1 eq '5'
                 || $tmp1 eq '6'
                 || $tmp1 eq '7'
                 || $tmp1 eq '8'
                 || $tmp1 eq '9'
                 || $tmp1 eq '10'
                 ) {
     
                # do not generate XML for empty originator fields 
                if (hasContent(param("origNamefirst" . $tmp1))) {    

            my $add = "";
            $add .= "<individualName>\n";
            $add .= "<givenName>";
            $add .= param("origNamefirst" . $tmp1);
            $add .= "</givenName>\n";
            $add .= "<surName>";
            $add .= param("origNamelast" . $tmp1);
            $add .= "</surName>\n";
            $add .= "</individualName>\n";
            
            if(param("origRole" . $tmp1) eq "Originator"){
            $creat .= "<creator>\n";
            $creat .= $add;
            $creat .= "</creator>\n";
            }
            elsif(param("origRole" . $tmp1) eq "Metadata Provider"){
            $metaP .= "<metadataProvider>\n";
            $metaP .= $add;
            $metaP .= "</metadataProvider>\n";
            }
            elsif((param("origRole" . $tmp1) eq "Publisher")  && ($publ eq "")){
            $publ .= "<publisher>\n";
            $publ .= $add;
            $publ .= "</publisher>\n";
            }
            else{
            $apart .= "<associatedParty>\n";
            $apart .= $add;
            $apart .= "<role>";
            $apart .= param("origRole" . $tmp1);
            $apart .= "</role>\n";
            $apart .= "</associatedParty>\n";
            }
        }
            }
        }
    }

    $creat .= "<creator>\n";
    $creat .= "<organizationName>$FORM::site</organizationName>\n";
    $creat .= "</creator>\n";

    $creat .= "<creator>\n";
    $creat .= "<organizationName>$organization</organizationName>\n";
    $creat .= "</creator>\n";

    if ($FORM::cfg eq 'nceas') {
        for (my $i = 0; $i < scalar(@FORM::wg); $i++) {
            $creat .= "<creator>\n";
            $creat .= "<organizationName>$FORM::wg[$i]</organizationName>\n";
            $creat .= "</creator>\n";
        }
    }
    $doc .= $creat;
    $doc .= $metaP;
    $doc .= $apart;

    $doc .= "<abstract>\n";
    $doc .= "<para>$FORM::abstract</para>\n";
    $doc .= "</abstract>\n";

    # Keyword information
    foreach my $tmp (param()) {
        if ($tmp =~ /keyword/) {
            my $tmp1 = $tmp;
            $tmp1 =~ s/keyword//; # get the index of the parameter 0, ..., 10
            if ( $tmp1 =~ /[0-9]/ ){
                # don't generate xml for empty keyword fields
                # don't generate taxonomic keyword fields, those go in taxonomic coverage
                if (hasContent(param($tmp))) {
                    $doc .= "<keywordSet>\n";
                    $doc .= "<keyword ";
                    if (hasContent(param("kwType" . $tmp1)) &&
                       (param("kwType" . $tmp1) !~ "none") ) {
                         $doc .= "keywordType=\"";
                         $doc .= param("kwType" . $tmp1);
                         $doc .= "\"";
                    }
                    $doc .= ">";
                    $doc .= param("keyword" . $tmp1);
                    $doc .= "</keyword>\n";
                    $doc .= "<keywordThesaurus>";
                    $doc .= param("kwTh" . $tmp1);
                    $doc .= "</keywordThesaurus>\n";
                    $doc .= "</keywordSet>\n";
                }
            }
        }
    }

    if (hasContent($FORM::addComments)) {
        $doc .= "<additionalInfo>\n";
        $doc .= "<para>$FORM::addComments</para>\n";
        $doc .= "</additionalInfo>\n";
    }

    if (hasContent($FORM::useConstraints) || 
        hasContent($FORM::useConstraintsOther)) {
        $doc .= "<intellectualRights>\n";
        if (hasContent($FORM::useConstraints)) {
            $doc .= "<para>$FORM::useConstraints</para>\n";
        }
        if (hasContent($FORM::useConstraintsOther)) {
            $doc .= "<para>$FORM::useConstraintsOther</para>\n";
        }
        $doc .= "</intellectualRights>\n";
    }

    
    if (hasContent($FORM::url)) {
    $doc .= "<distribution>\n";
        $doc .= "<online>\n";
    $doc .= "<url>$FORM::url</url>\n";
    $doc .= "</online>\n";
    $doc .= "</distribution>\n";
    }
    
    $doc .= "<distribution>\n";
    $doc .= "<offline>\n";
    $doc .= "<mediumName>" . "$FORM::dataMedium   $FORM::dataMediumOther";
    $doc .= "</mediumName>\n";
    $doc .= "</offline>\n";
    $doc .= "</distribution>\n";
            
    $doc .= "<coverage>\n";
    $doc .= "<temporalCoverage>\n";


    if (hasContent($FORM::endingYear)) {
    $doc .= "<rangeOfDates>\n";
    if (hasContent($FORM::beginningMonth)) {
        my $month = ("JAN","FEB","MAR","APR","MAY","JUN",
             "JUL","AUG","SEP","OCT","NOV","DEC")
        [$FORM::beginningMonth - 1];
        $doc .= "<beginDate>\n";
        $doc .= "<calendarDate>";
        $doc .= "$FORM::beginningYear-$FORM::beginningMonth-$FORM::beginningDay";
        $doc .= "</calendarDate>\n";
        $doc .= "</beginDate>\n";
    } else {
        $doc .= "<beginDate>\n";
        $doc .= "<calendarDate>";
        $doc .= "$FORM::beginningYear";
        $doc .= "</calendarDate>\n";
        $doc .= "</beginDate>\n";
    }

    if (hasContent($FORM::endingMonth)) {
        my $month = ("JAN","FEB","MAR","APR","MAY","JUN",
             "JUL","AUG","SEP","OCT","NOV","DEC")
        [$FORM::endingMonth - 1];
        $doc .= "<endDate>\n";
        $doc .= "<calendarDate>";
        $doc .= "$FORM::endingYear-$FORM::endingMonth-$FORM::endingDay";
        $doc .= "</calendarDate>\n";
        $doc .= "</endDate>\n";
    } else {
        $doc .= "<endDate>\n";
        $doc .= "<calendarDate>";
        $doc .= "$FORM::endingYear";
        $doc .= "</calendarDate>\n";
        $doc .= "</endDate>\n";
    }
    $doc .= "</rangeOfDates>\n";
    } else {
    $doc .= "<singleDateTime>\n";
    if (hasContent($FORM::beginningMonth)) {
        my $month = ("JAN","FEB","MAR","APR","MAY","JUN",
             "JUL","AUG","SEP","OCT","NOV","DEC")
        [$FORM::beginningMonth - 1];
        $doc .= "<calendarDate>";
        $doc .= "$FORM::beginningYear-$FORM::beginningMonth-$FORM::beginningDay";
        $doc .= "</calendarDate>\n";
    } else {
        $doc .= "<calendarDate>";
        $doc .= "$FORM::beginningYear";
        $doc .= "</calendarDate>\n";
    }
    $doc .= "</singleDateTime>\n";
    }

    $doc .= "</temporalCoverage>\n";
    
    $doc .= "<geographicCoverage>\n";
    $doc .= "<geographicDescription>$FORM::geogdesc</geographicDescription>\n";
    $doc .= "<boundingCoordinates>\n";

    # if the second latitude is missing, then set the second lat/long pair 
    # equal to the first this makes a point appear like a rectangle 
    if ($FORM::latDeg2 == 0 && $FORM::latMin2 == 0 && $FORM::latSec2 == 0) {
    
        $latDeg2 = $latDeg1;
        $latMin2 = $latMin1;
        $latSec2 = $latSec1;
        $hemisphLat2 = $hemisphLat1;
        $longDeg2 = $longDeg1;
        $longMin2 = $longMin1;
        $longSec2 = $longSec1;
        $hemisphLong2 = $hemisphLong1;
    }
    else
    {
        $latDeg2 = $FORM::latDeg2;
        $latMin2 = $FORM::latMin2;
        $latSec2 = $FORM::latSec2;
        $hemisphLat2 = $FORM::hemisphLat2;
        $longDeg2 = $FORM::longDeg2;
        $longMin2 = $FORM::longMin2;
        $longSec2 = $FORM::longSec2;
        $hemisphLong2 = $FORM::hemisphLong2;
    } 
    
   
    my $hemisph;
    $hemisph = ($hemisphLong1 eq "W") ? -1 : 1;
    $doc .= "<westBoundingCoordinate>";
    $doc .= $hemisph * ($longDeg1 + (60*$longMin1+$longSec1)/3600);
    $doc .= "</westBoundingCoordinate>\n";

    $hemisph = ($hemisphLong2 eq "W") ? -1 : 1;
    $doc .= "<eastBoundingCoordinate>";
    $doc .= $hemisph * ($longDeg2 + (60*$longMin2+$longSec2)/3600);
    $doc .= "</eastBoundingCoordinate>\n";

    $hemisph = ($hemisphLat1 eq "S") ? -1 : 1;
    $doc .= "<northBoundingCoordinate>";
    $doc .= $hemisph * ($latDeg1 + (60*$latMin1+$latSec1)/3600);
    $doc .= "</northBoundingCoordinate>\n";

    $hemisph = ($hemisphLat2 eq "S") ? -1 : 1;
    $doc .= "<southBoundingCoordinate>";
    $doc .= $hemisph * ($latDeg2 + (60*$latMin2+$latSec2)/3600);
    $doc .= "</southBoundingCoordinate>\n";

    $doc .= "</boundingCoordinates>\n";
    $doc .= "</geographicCoverage>\n";

    # Write out the taxonomic coverage fields
    my $foundFirstTaxon = 0;
    foreach my $trn (param()) {
        if ($trn =~ /taxonRankName/) {
            my $taxIndex = $trn;
            $taxIndex =~ s/taxonRankName//; # get the index of the parameter 0, ..., 10
            my $trv = "taxonRankValue".$taxIndex;
            if ( $taxIndex =~ /[0-9]/ ){
                if (hasContent(param($trn)) && hasContent(param($trv))) {
                    if (! $foundFirstTaxon) {
                        $doc .= "<taxonomicCoverage>\n";
                        $foundFirstTaxon = 1;
                        if (hasContent($FORM::taxaAuth)) {
                            $doc .= "<generalTaxonomicCoverage>".$FORM::taxaAuth."</generalTaxonomicCoverage>\n";
                        }
                    }
                    $doc .= "<taxonomicClassification>\n";
                    $doc .= "  <taxonRankName>".param($trn)."</taxonRankName>\n";
                    $doc .= "  <taxonRankValue>".param($trv)."</taxonRankValue>\n";
                    $doc .= "</taxonomicClassification>\n";
                }
            }
        }
    }
    if ($foundFirstTaxon) {
        $doc .= "</taxonomicCoverage>\n";
    }

    $doc .= "</coverage>\n";

    $doc .= $cont;
    $doc .= $publ;
    
    if ((hasContent($FORM::methodTitle)) || scalar(@FORM::methodsPara) > 0) {
        my $methods = "<methods><methodStep><description><section>\n";
        if (hasContent($FORM::methodTitle)) {
            $methods .= "<title>$FORM::methodTitle</title>\n";
        }
        for (my $i = 0; $i < scalar(@FORM::methodPara); $i++) {
            $methods .= "<para>$FORM::methodPara[$i]</para>\n";
        }
        $methods .= "</section></description></methodStep>\n";
        if (hasContent($FORM::studyExtentDescription)) {
            $methods .= "<sampling><studyExtent><description>\n";
            $methods .= "<para>$FORM::studyExtentDescription</para>\n";
            $methods .= "</description></studyExtent>\n";
            $methods .= "<samplingDescription>\n";
            $methods .= "<para>$FORM::samplingDescription</para>\n";
            $methods .= "</samplingDescription>\n";
            $methods .= "</sampling>\n";
        }
        $methods .= "</methods>\n";
        $doc .= $methods;
    }

    $doc .= "<access authSystem=\"knb\" order=\"denyFirst\">\n";
    $doc .= "<allow>\n";
    $doc .= "<principal>$username</principal>\n";
    $doc .= "<permission>all</permission>\n";
    $doc .= "</allow>\n";
    $doc .= "<allow>\n";
    $doc .= "<principal>uid=$FORM::username,o=$FORM::organization,dc=ecoinformatics,dc=org</principal>\n";
    $doc .= "<permission>all</permission>\n";
    $doc .= "</allow>\n";
    $doc .= "<allow>\n";
    $doc .= "<principal>public</principal>\n";
    $doc .= "<permission>read</permission>\n";
    $doc .= "</allow>\n";
    $doc .= "</access>\n";
    
    $doc .= "</dataset>\n</eml:eml>\n";

    return $doc;
}


################################################################################
# 
# send an email message notifying the moderator of a new submission 
#
################################################################################
sub sendNotification {
    my $identifier = shift;
    my $mailhost = shift;
    my $sender = shift;
    my $recipient = shift;

    my $smtp = Net::SMTP->new($mailhost);
    $smtp->mail($sender);
    $smtp->to($recipient);

    my $message = <<"    ENDOFMESSAGE";
    To: $recipient
    From: $sender
    Subject: New data submission
    
    Data was submitted to the data registry.  
    The identifying information for the new data set is:

    Identifier: $identifier
    Title: $FORM::title
    Submitter: $FORM::providerGivenName $FORM::providerSurName

    Please review the submmission and grant public read access if appropriate.
    Thanks
    
    ENDOFMESSAGE
    $message =~ s/^[ \t\r\f]+//gm;

    $smtp->data($message);
    $smtp->quit;
}


################################################################################
# 
# read the eml document and send back a form with values filled in. 
#
################################################################################
sub modifyData {
    
    # create metacat instance
    my $metacat;
    my $docid = $FORM::docid;
    my $httpMessage;
    my $doc;
    my $xmldoc;
    my $findType;
    my $parser = XML::LibXML->new();
    my @fileArray;
    my $pushDoc;
    my $alreadyInArray;
    my $node;
    my $response; 
    my $element;
    my $tempfile;

    $metacat = Metacat->new();
    if ($metacat) {
        $metacat->set_options( metacatUrl => $metacatUrl );
    } else {
        #die "failed during metacat creation\n";
        push(@errorMessages, "Failed during metacat creation.");
    }
    
    $httpMessage = $metacat->read($docid);
    $doc = $httpMessage->content();
    $xmldoc = $parser->parse_string($doc);

    #$tempfile = $xslConvDir.$docid;
    #push (@fileArray, $tempfile);

    if ($xmldoc eq "") {
        $error ="Error in parsing the eml document";
        push(@errorMessages, $error);
    } else {
        $findType = $xmldoc->findnodes('//dataset/identifier');
        if ($findType->size() > 0) {
            # This is a eml beta6 document
            # Read the documents mentioned in triples also
        
            $findType = $xmldoc->findnodes('//dataset/triple');
            if ($findType->size() > 0) {
                foreach $node ($findType->get_nodelist) {
                    $pushDoc = findValue($node, 'subject');
            
                    # If the file is already in @fileArray then do not add it 
                    $alreadyInArray = 0;
                    foreach $element (@fileArray) {
                        $tempfile = $tmpdir."/".$pushDoc;
                        if ($element eq $pushDoc) {
                            $alreadyInArray = 1;
                        }
                    }
            
                    if (!$alreadyInArray) {
                        $tempfile = $tmpdir."/".$pushDoc;
                        $response = "";
                        $response = $metacat->read($pushDoc);    
                        if (! $response) {
                            # could not read
                            #push(@errorMessages, $response);
                            push(@errorMessages, $metacat->getMessage());
                            push(@errorMessages, "Failed during reading.\n");
                        } else {
                            my $xdoc = $response->content();
                            #$tempfile = $xslConvDir.$pushDoc;
                            open (TFILE,">$tempfile") || 
                                die ("Cant open xml file... $tempfile\n");
                            print TFILE $xdoc;
                            close(TFILE);
                            push (@fileArray, $tempfile);
                        }
                    }
                }
            }

            # Read the main document. 

            $tempfile = $tmpdir."/".$docid; #= $xslConvDir.$docid;
            open (TFILE,">$tempfile") || die ("Cant open xml file...\n");
            print TFILE $doc;
            close(TFILE);
        
            # Transforming beta6 to eml 2
            my $xslt;
            my $triplesheet;
            my $results;
            my $stylesheet;
            my $resultsheet;
        
            $xslt = XML::LibXSLT->new();
            #$tempfile = $xslConvDir."triple_info.xsl";
            $tempfile = $tmpdir."/"."triple_info.xsl";
    
            $triplesheet = $xslt->parse_stylesheet_file($tempfile);

            #$results = $triplesheet->transform($xmldoc, packageDir => "\'$tmpdir/\'", packageName => "\'$docid\'");
            $results = $triplesheet->transform($xmldoc, packageDir => "\'$tmpdir/\'", packageName => "\'$docid\'");

            #$tempfile = $xslConvDir."emlb6toeml2.xsl";
            $tempfile = $tmpdir."/"."emlb6toeml2.xsl";
            $stylesheet = $xslt->parse_stylesheet_file($tempfile);
            $resultsheet = $stylesheet->transform($results);
        
            #$tempfile = "/usr/local/apache2/htdocs/xml/test.xml";;
            #open (TFILE,">$tempfile") || die ("Cant open xml file...\n");
            #print TFILE $stylesheet->output_string($resultsheet);
            #close(TFILE);

            getFormValuesFromEml2($resultsheet);
            
            # Delete the files written earlier. 
            unlink @fileArray;

        } else {
            getFormValuesFromEml2($xmldoc);
        }
    }   
    
    if (scalar(@errorMessages)) {
        # if any errors, print them in the response template 
        $$templateVars{'status'} = 'failure';
        $$templateVars{'errorMessages'} = \@errorMessages;
        $error = 1;
        $$templateVars{'function'} = "modification";
        $$templateVars{'section'} = "Modification Status";
        $template->process( $responseTemplate, $templateVars); 
    } else {
        $$templateVars{'form'} = 're_entry';
        $template->process( $entryFormTemplate, $templateVars);
    }
}

################################################################################
# 
# Parse an EML 2.0.0 file and extract the metadata into perl variables for 
# processing and returning to the template processor
#
################################################################################
sub getFormValuesFromEml2 {
    
    my $doc = shift;
    my $results;
    my $error;
    my $node;
    my $tempResult;
    my $tempNode;
    my $aoCount = 1;
    my $foundDSO;

    # find out the tag <alternateIdentifier>. 
    $results = $doc->findnodes('//dataset/alternateIdentifier');
    if ($results->size() > 1) {
        errMoreThanOne("alternateIdentifier");
    } else {
        foreach $node ($results->get_nodelist) {
            $$templateVars{'identifier'} = findValue($node, '../alternateIdentifier');
        }
    }

    # find out the tag <title>. 
    $results = $doc->findnodes('//dataset/title');
    if ($results->size() > 1) {
        errMoreThanOne("title");
    } elsif ($results->size() < 1) {
        $error ="Following tag not found: title. Please use Morpho to edit this document";
        push(@errorMessages, $error."\n");
        #if ($DEBUG == 1){ print $error;}
    } else {
        foreach $node ($results->get_nodelist) {
            $$templateVars{'title'} = findValue($node, '../title');
        }
    }

    # find out the tag <creator>. 
    $results = $doc->findnodes('//dataset/creator/individualName');
    debug("Registry: Creators: ".$results->size());
    if ($results->size() > 11) {
        errMoreThanN("creator/individualName");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "../positionName|../onlineURL|../userId", 
              "positionName, onlineURL, userId");
        
            dontOccur($node, "./saluation", "saluation");                
        
            debug("Registry: Checking a creator in loop 1...");
            $tempResult = $node->findnodes('../address|../phone|../electronicmailAddress|../organizationName');
            if($tempResult->size > 0) {
                if($foundDSO == 0) {
                    $foundDSO = 1;
     
                    debug("Registry: Recording a creator in loop 1...");
                    $$templateVars{'origNamefirst0'} = findValue($node, 'givenName');
                    $$templateVars{'origNamelast0'} = findValue($node, 'surName');
            
                    my $tempResult2 = $node->findnodes('../address');
                    if ($tempResult2->size > 1) {
                        errMoreThanOne("address");
                    } else {
                        foreach my $tempNode2 ($tempResult2->get_nodelist) {
                            $$templateVars{'origDelivery'} = findValue($tempNode2, 'deliveryPoint');
                            $$templateVars{'origCity'} = findValue($tempNode2, 'city');
                            $$templateVars{'origState'} = findValue($tempNode2, 'administrativeArea');
                            $$templateVars{'origZIP'} = findValue($tempNode2, 'postalCode');
                            $$templateVars{'origCountry'} = findValue($tempNode2, 'country');
                        }
                    }
            
                    my $tempResult3 = $node->findnodes('../phone');
                    if ($tempResult3->size > 2) {
                        errMoreThanN("phone");
                    } else {
                        foreach my $tempNode2 ($tempResult3->get_nodelist) {
                            if ($tempNode2->hasAttributes()) {
                                my @attlist = $tempNode2->attributes();
                                if ($attlist[0]->value eq "Fax") {
                                    $$templateVars{'origFAX'} = $tempNode2->textContent();
                                } else {
                                    $$templateVars{'origPhone'} = $tempNode2->textContent();
                                }
                            } else {
                                $$templateVars{'origPhone'} = $tempNode2->textContent();
                            }
                        }
                    }
                    $$templateVars{'origEmail'} = findValue($node, '../electronicMailAddress');
                    $$templateVars{'origNameOrg'} = findValue($node, '../organizationName');
                } else {
                    errMoreThanN("address, phone and electronicMailAddress");
                }
            }
        }
        foreach $node ($results->get_nodelist) {
            debug("Registry: Checking a creator in loop 2...");
            $tempResult = $node->findnodes('../address|../phone|../electronicmailAddress|../organizationName');
            if ($tempResult->size == 0) {
                if ($foundDSO == 0) {
                    debug("Registry: Recording a creator in loop 2 block A...");
                    $foundDSO = 1;
                    $$templateVars{'origNamefirst0'} = findValue($node, 'givenName');
                    $$templateVars{'origNamelast0'} = findValue($node, 'surName');
                    $$templateVars{'origNameOrg'} = findValue($node, '../organizationName');
                } else {
                    debug("Registry: Recording a creator in loop 2 block B...");
                    $$templateVars{"origNamefirst$aoCount"} =  findValue($node, './givenName');
                    $$templateVars{"origNamelast$aoCount"} =  findValue($node, './surName');
                    $$templateVars{"origRole$aoCount"} = "Originator";
                    $aoCount++;
                }
            }
        }
    }

    $results = $doc->findnodes('//dataset/creator/organizationName');
    my $wgroups = $doc->findnodes("//dataset/creator/organizationName[contains(text(),'(NCEAS ')]");
    debug("Registry: Number Org: ".$results->size());
    debug("Registry:  Number WG: ".$wgroups->size());
    if ($results->size() - $wgroups->size() > 3) {
        errMoreThanN("creator/organizationName");    
    } else {
        foreach $node ($results->get_nodelist) {
            my $tempValue = findValue($node,'../organizationName');
            $tempResult = $node->findnodes('../individualName');
            if ($tempResult->size == 0 && $tempValue ne $organization) {
                $$templateVars{'site'} = $tempValue;
            }
        }
        if ($FORM::cfg eq 'nceas') {
            my @wg;
            foreach $node ($results->get_nodelist) {
                my $tempValue = findValue($node,'../organizationName');
                $wg[scalar(@wg)] = $tempValue;
            }
            my $projects = getProjectList();
            $$templateVars{'projects'} = $projects;
            $$templateVars{'wg'} = \@wg;
        }
    }

    $results = $doc->findnodes('//dataset/metadataProvider');
    if ($results->size() > 11) {
        errMoreThanN("metadataProvider");    
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "./organizationName|./positionName|./onlineURL|./userId|./electronicMailAddress|./phone|./address", 
                "organizationName, positionName, onlineURL, userId, electronicMailAddress, phone, address in metadataProvider");
        
            $tempResult = $node->findnodes('./individualName');
            if ($tempResult->size > 1) {
                errMoreThanOne("metadataProvider/indvidualName");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    if ($$templateVars{'providerGivenName'} ne "") {
                        $$templateVars{"origNamefirst$aoCount"} =  findValue($tempNode, './givenName');
                        $$templateVars{"origNamelast$aoCount"} =  findValue($tempNode, './surName');
                        $$templateVars{"origRole$aoCount"} = "Metadata Provider";
                        $aoCount++;
                    } else {
                        $$templateVars{'providerGivenName'} =  findValue($tempNode, './givenName');
                        $$templateVars{'providerSurName'} =  findValue($tempNode, './surName');
                    }
                }
            }
        }
    }

    $results = $doc->findnodes('//dataset/associatedParty');
    if ($results->size() > 10) {
        errMoreThanN("associatedParty");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "./organizationName|./positionName|./onlineURL|./userId|./electronicMailAddress|./phone|./address", 
                "organizationName, positionName, onlineURL, userId, electronicMailAddress, phone, address in associatedParty");
       
            $tempResult = $node->findnodes('./individualName');
            if ($tempResult->size > 1) {
                errMoreThanOne("associatedParty/indvidualName");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    $$templateVars{"origNamefirst$aoCount"} =  findValue($tempNode, './givenName');
                    $$templateVars{"origNamelast$aoCount"} =  findValue($tempNode, './surName');
                    $$templateVars{"origRole$aoCount"} = findValue($tempNode, '../role');
                    $aoCount++;           
                }
            }
        }
    }

    $results = $doc->findnodes('//dataset/publisher');
    if ($results->size() > 10) {
        errMoreThanN("publisher");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "./organizationName|./positionName|./onlineURL|./userId|./electronicMailAddress|./phone|./address", 
                "organizationName, positionName, onlineURL, userId, electronicMailAddress, phone, address in associatedParty");
       
            $tempResult = $node->findnodes('./individualName');
            if ($tempResult->size > 1) {
                errMoreThanOne("publisher/indvidualName");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    $$templateVars{"origNamefirst$aoCount"} =  findValue($tempNode, './givenName');
                    $$templateVars{"origNamelast$aoCount"} =  findValue($tempNode, './surName');
                    $$templateVars{"origRole$aoCount"} = "Publisher";
                    $aoCount++;           
                }
            }
        }
    }

    if ($aoCount > 11) {
        errMoreThanN("Additional Originators");
    } 

    dontOccur($doc, "./pubDate", "pubDate");
    dontOccur($doc, "./language", "language");
    dontOccur($doc, "./series", "series");

    $results = $doc->findnodes('//dataset/abstract');
    if ($results->size() > 1) {
        errMoreThanOne("abstract");
    } else {
        foreach my $node ($results->get_nodelist) {
            dontOccur($node, "./section", "section");
            $$templateVars{'abstract'} = findValueNoChild($node, "para");
        }
    }

    $results = $doc->findnodes('//dataset/keywordSet');
    if ($results->size() > 10) {
        errMoreThanN("keywordSet");
    } else {
        my $count = 0;
        foreach $node ($results->get_nodelist) {
            $tempResult = $node->findnodes('./keyword');
            if ($tempResult->size() > 1) {
                errMoreThanOne("keyword");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    $$templateVars{"keyword$count"} = $tempNode->textContent();
                    if ($tempNode->hasAttributes()) {
                        my @attlist = $tempNode->attributes();
                        $$templateVars{"kwType$count"} = $attlist[0]->value;
                    }  
                 } 
            }
            $$templateVars{"kwTh$count"} = findValue($node, "keywordThesaurus");
            $count++;
        }
    
        while ($count<11) {
            $$templateVars{"kwType$count"} = "none";
            $$templateVars{"kwTh$count"} = "none";
            $count++;
        }
    }

    $results = $doc->findnodes('//dataset/additionalInfo');
    if ($results->size() > 1) {
        errMoreThanOne("additionalInfo");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "./section", "section");
            $$templateVars{'addComments'} = findValueNoChild($node, "para");
        }
    }

    $$templateVars{'useConstraints'} = "";
    $results = $doc->findnodes('//dataset/intellectualRights');
    if ($results->size() > 1) {
        errMoreThanOne("intellectualRights");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "./section", "section in intellectualRights");

            $tempResult = $node->findnodes("para");
            if ($tempResult->size > 2) {
                   errMoreThanN("para");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    my $childNodes = $tempNode->childNodes;
                    if ($childNodes->size() > 1) {
                        $error ="The tag para in intellectualRights has children which cannot be shown using the form. Please use Morpho to edit this document";    
                        push(@errorMessages, $error);
                        #if ($DEBUG == 1){ print $error."\n";}
                    } else {
                        #print $tempNode->nodeName().":".$tempNode->textContent();
                        #print "\n";
                        if ($$templateVars{'useConstraints'} eq "") {
                            $$templateVars{'useConstraints'} = $tempNode->textContent();
                        } else {
                            $$templateVars{'useConstraintsOther'} = $tempNode->textContent();
                        }
                    }
                }
            }
        }
    }

    $results = $doc->findnodes('//dataset/distribution/online');
    if ($results->size() > 1) {
        errMoreThanOne("distribution/online");
    } else {
        foreach my $tempNode ($results->get_nodelist){
            $$templateVars{'url'} = findValue($tempNode, "url");
            dontOccur($tempNode, "./connection", "/distribution/online/connection");
            dontOccur($tempNode, "./connectionDefinition", "/distribution/online/connectionDefinition");
        }
    }

    $results = $doc->findnodes('//dataset/distribution/offline');
    if ($results->size() > 1) {
        errMoreThanOne("distribution/online");
    } else {
        foreach my $tempNode ($results->get_nodelist) {
            $$templateVars{'dataMedium'} = findValue($tempNode, "mediumName");
            dontOccur($tempNode, "./mediumDensity", "/distribution/offline/mediumDensity");
            dontOccur($tempNode, "./mediumDensityUnits", "/distribution/offline/mediumDensityUnits");
            dontOccur($tempNode, "./mediumVolume", "/distribution/offline/mediumVolume");
            dontOccur($tempNode, "./mediumFormat", "/distribution/offline/mediumFormat");
            dontOccur($tempNode, "./mediumNote", "/distribution/offline/mediumNote");
        }
    }

    dontOccur($doc, "./inline", "//dataset/distribution/inline");

    $results = $doc->findnodes('//dataset/coverage');
    if ($results->size() > 1) {
        errMoreThanOne("coverage");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "./temporalCoverage/rangeOfDates/beginDate/time|./temporalCoverage/rangeOfDates/beginDate/alternativeTimeScale|./temporalCoverage/rangeOfDates/endDate/time|./temporalCoverage/rangeOfDates/endDate/alternativeTimeScale|./taxonomicCoverage/taxonomicSystem|./taxonomicCoverage/taxonomicClassification/commonName|./taxonomicCoverage/taxonomicClassification/taxonomicClassification|./geographicCoverage/datasetGPolygon|./geographicCoverage/boundingCoordinates/boundingAltitudes", "temporalCoverage/rangeOfDates/beginDate/time, /temporalCoverage/rangeOfDates/beginDate/alternativeTimeScale, /temporalCoverage/rangeOfDates/endDate/time, /temporalCoverage/rangeOfDates/endDate/alternativeTimeScale, /taxonomicCoverage/taxonomicSystem, /taxonomicCoverage/taxonomicClassification/commonName, /taxonomicCoverage/taxonomicClassification/taxonomicClassification, /geographicCoverage/datasetGPolygon, /geographicCoverage/boundingCoordinates/boundingAltitudes");

            $tempResult = $node->findnodes('./temporalCoverage');
            if ($tempResult->size > 1) {
                   errMoreThanOne("temporalCoverage");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    my $x;
                    my $y;
                    my $z;
                    my $tempdate = findValue($tempNode, "rangeOfDates/beginDate/calendarDate");
                    ($x, $y, $z) = split("-", $tempdate); 
                    $$templateVars{'beginningYear'} = $x;
                    $$templateVars{'beginningMonth'} = $y;
                    $$templateVars{'beginningDay'} = $z;
    
                    $tempdate = findValue($tempNode, "rangeOfDates/endDate/calendarDate");
                    ($x, $y, $z) = split("-", $tempdate);
                    $$templateVars{'endingYear'} = $x;
                    $$templateVars{'endingMonth'} = $y;
                    $$templateVars{'endingDay'} = $z;
                
                    $tempdate = "";
                    $tempdate = findValue($tempNode, "singleDateTime/calendarDate");
                    if($tempdate ne ""){
                        ($x, $y, $z) = split("-", $tempdate);
                        $$templateVars{'beginningYear'} = $x;
                        $$templateVars{'beginningMonth'} = $y;
                        $$templateVars{'beginningDay'} = $z;
                    }  
                }
            }

            $tempResult = $node->findnodes('./geographicCoverage');
            if ($tempResult->size > 1) {
                errMoreThanOne("geographicCoverage");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    my $geogdesc = findValue($tempNode, "geographicDescription");
                    debug("Registry: geogdesc from xml is: $geogdesc");
                    $$templateVars{'geogdesc'} = $geogdesc;
                    my $coord = findValue($tempNode, "boundingCoordinates/westBoundingCoordinate");
                    if ($coord > 0) {
                        #print "+";
                        $$templateVars{'hemisphLong1'} = "E";
                    } else {
                        #print "-";
                        eval($coord = $coord * -1);
                        $$templateVars{'hemisphLong1'} = "W";
                    }
                    eval($$templateVars{'longDeg1'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'longMin1'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'longSec1'} = int($coord));
                    
                    $coord = findValue($tempNode, "boundingCoordinates/southBoundingCoordinate");
                    if ($coord > 0) {
                        #print "+";
                        $$templateVars{'hemisphLat2'} = "N";
                    } else {
                        #print "-";
                        eval($coord = $coord * -1);
                        $$templateVars{'hemisphLat2'} = "S";
                    }
                    eval($$templateVars{'latDeg2'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'latMin2'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'latSec2'} = int($coord));
        
                    $coord = findValue($tempNode, "boundingCoordinates/northBoundingCoordinate");
                    if ($coord > 0) {
                        #print "+";
                        $$templateVars{'hemisphLat1'} = "N";
                    } else {
                        #print "-";
                        eval($coord = $coord * -1);
                        $$templateVars{'hemisphLat1'} = "S";
                    }
                    eval($$templateVars{'latDeg1'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'latMin1'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'latSec1'} = int($coord));
        
                    $coord = findValue($tempNode, "boundingCoordinates/eastBoundingCoordinate");
                    if ($coord > 0) {
                        #print "+";
                        $$templateVars{'hemisphLong2'} = "E";
                    } else {
                        #print "-";
                        eval($coord = $coord * -1);
                        $$templateVars{'hemisphLong2'} = "W";
                    }
                    eval($$templateVars{'longDeg2'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'longMin2'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'longSec2'} = int($coord));
                }
            }

            $tempResult = $node->findnodes('./taxonomicCoverage/taxonomicClassification');
            my $taxonIndex = 0;
            foreach $tempNode ($tempResult->get_nodelist) {
                $taxonIndex++;
                my $taxonRankName = findValue($tempNode, "taxonRankName");
                my $taxonRankValue = findValue($tempNode, "taxonRankValue");
                $$templateVars{"taxonRankName".$taxonIndex} = $taxonRankName;
                $$templateVars{"taxonRankValue".$taxonIndex} = $taxonRankValue;
            }
            $$templateVars{'taxaCount'} = $taxonIndex;
            my $taxaAuth = findValue($node, "./taxonomicCoverage/generalTaxonomicCoverage");
            $$templateVars{'taxaAuth'} = $taxaAuth;
        }
    }
    dontOccur($doc, "./purpose", "purpose");
    dontOccur($doc, "./maintenance", "maintnance");

    $results = $doc->findnodes('//dataset/contact/individualName');
    if ($results->size() > 1) {
        errMoreThanOne("contact/individualName");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "../positionName|../onlineURL|../userId", 
              "positionName, onlineURL, userId in contact tag");
            dontOccur($node, "./saluation", "saluation in contact tag");                
        
            $tempResult = $node->findnodes('../address|../phone|../electronicmailAddress|../organizationName');
            if ($tempResult->size > 0) {
                $$templateVars{'origNamefirstContact'} = findValue($node, 'givenName');
                $$templateVars{'origNamelastContact'} = findValue($node, 'surName');
    
                my $tempResult2 = $node->findnodes('../address');
                if ($tempResult2->size > 1) {
                    errMoreThanOne("address");
                } else {
                    foreach my $tempNode2 ($tempResult2->get_nodelist) {
                        $$templateVars{'origDeliveryContact'} = findValue($tempNode2, 'deliveryPoint');
                        $$templateVars{'origCityContact'} = findValue($tempNode2, 'city');
                        $$templateVars{'origStateContact'} = findValue($tempNode2, 'administrativeArea');
                        $$templateVars{'origZIPContact'} = findValue($tempNode2, 'postalCode');
                        $$templateVars{'origCountryContact'} = findValue($tempNode2, 'country');
                    }
                }
            
                my $tempResult3 = $node->findnodes('../phone');
                if ($tempResult3->size > 2) {
                    errMoreThanN("phone");
                } else {
                    foreach my $tempNode2 ($tempResult3->get_nodelist) {
                        if ($tempNode2->hasAttributes()) {
                            my @attlist = $tempNode2->attributes();
                            if ($attlist[0]->value eq "Fax") {
                                $$templateVars{'origFAXContact'} = $tempNode2->textContent();
                            } else {
                                $$templateVars{'origPhoneContact'} = $tempNode2->textContent();
                            }
                        } else {
                            $$templateVars{'origPhoneContact'} = $tempNode2->textContent();
                        }
                    }
                }
                $$templateVars{'origEmailContact'} = findValue($node, '../electronicMailAddress');
                $$templateVars{'origNameOrgContact'} = findValue($node, '../organizationName');
            } else {
                $$templateVars{'origNamefirstContact'} = findValue($node, 'givenName');
                $$templateVars{'origNamelastContact'} = findValue($node, 'surName');
                $$templateVars{'origNameOrgContact'} = findValue($node, '../organizationName');
            }
        }
    }
    
    $results = $doc->findnodes(
            '//dataset/methods/methodStep/description/section');
    debug("Registry: Number methods: ".$results->size());
    if ($results->size() > 1) {
        errMoreThanN("methods/methodStep/description/section");    
    } else {

        my @methodPara;
        foreach $node ($results->get_nodelist) {
            my @children = $node->childNodes;
            for (my $i = 0; $i < scalar(@children); $i++) {
                debug("Registry: Method child loop ($i)");
                my $child = $children[$i];
                if ($child->nodeName eq 'title') {
                    my $title = $child->textContent();
                    debug("Registry: Method title ($title)");
                    $$templateVars{'methodTitle'} = $title;
                } elsif ($child->nodeName eq 'para') {
                    my $para = $child->textContent();
                    debug("Registry: Method para ($para)");
                    $methodPara[scalar(@methodPara)] = $para;
                }
            }
        }
        if (scalar(@methodPara) > 0) {
            $$templateVars{'methodPara'} = \@methodPara;
        }
    }

    $results = $doc->findnodes(
            '//dataset/methods/sampling/studyExtent/description/para');
    if ($results->size() > 1) {
        errMoreThanN("methods/sampling/studyExtent/description/para");    
    } else {
        foreach $node ($results->get_nodelist) {
            my $studyExtentDescription = $node->textContent();
            $$templateVars{'studyExtentDescription'} = $studyExtentDescription;
        }
    }

    $results = $doc->findnodes(
            '//dataset/methods/sampling/samplingDescription/para');
    if ($results->size() > 1) {
        errMoreThanN("methods/sampling/samplingDescription/para");    
    } else {
        foreach $node ($results->get_nodelist) {
            my $samplingDescription = $node->textContent();
            $$templateVars{'samplingDescription'} = $samplingDescription;
        }
    }

    dontOccur($doc, "//methodStep/citation", "methodStep/citation");
    dontOccur($doc, "//methodStep/protocol", "methodStep/protocol");
    dontOccur($doc, "//methodStep/instrumentation", "methodStep/instrumentation");
    dontOccur($doc, "//methodStep/software", "methodStep/software");
    dontOccur($doc, "//methodStep/subStep", "methodStep/subStep");
    dontOccur($doc, "//methodStep/dataSource", "methodStep/dataSource");
    dontOccur($doc, "//methods/qualityControl", "methods/qualityControl");

    dontOccur($doc, "//methods/sampling/spatialSamplingUnits", "methods/sampling/spatialSamplingUnits");
    dontOccur($doc, "//methods/sampling/citation", "methods/sampling/citation");
    dontOccur($doc, "./pubPlace", "pubPlace");
    dontOccur($doc, "./project", "project");
    
    dontOccur($doc, "./dataTable", "dataTable");
    dontOccur($doc, "./spatialRaster", "spatialRaster");
    dontOccur($doc, "./spatialVector", "spatialVector");
    dontOccur($doc, "./storedProcedure", "storedProcedure");
    dontOccur($doc, "./view", "view");
    dontOccur($doc, "./otherEntity", "otherEntity");
    dontOccur($doc, "./references", "references");
    
    dontOccur($doc, "//citation", "citation");
    dontOccur($doc, "//software", "software");
    dontOccur($doc, "//protocol", "protocol");
    dontOccur($doc, "//additionalMetadata", "additionalMetadata");    
}

################################################################################
# 
# Delete the eml file that has been requested for deletion. 
#
################################################################################
sub deleteData {
    my $deleteAll = shift;
    
    # create metacat instance
    my $metacat;
    my $docid = $FORM::docid;
    
    $metacat = Metacat->new();
    if ($metacat) {
        $metacat->set_options( metacatUrl => $metacatUrl );
    } else {
        #die "failed during metacat creation\n";
        push(@errorMessages, "Failed during metacat creation.");
    }

    # Login to metacat
    my $userDN = $FORM::username;
    my $userOrg = $FORM::organization;
    my $userPass = $FORM::password;
    my $dname = "uid=$userDN,o=$userOrg,dc=ecoinformatics,dc=org";
    
    my $errorMessage = "";
    my $response = $metacat->login($dname, $userPass);

    if (! $response) {
    # Could not login
        push(@errorMessages, $metacat->getMessage());
        push(@errorMessages, "Failed during login.\n");

    } else {
    #Able to login - try to delete the file    

    my $parser;
    my @fileArray;
    my $httpMessage;
    my $xmldoc;
    my $doc;
    my $pushDoc;
    my $alreadyInArray;
    my $findType;
        my $node;
    my $response; 
    my $element;

    push (@fileArray, $docid);
    $parser = XML::LibXML->new();

        $httpMessage = $metacat->read($docid);
    $doc = $httpMessage->content();    
    $xmldoc = $parser->parse_string($doc);

    if ($xmldoc eq "") {
        $error ="Error in parsing the eml document";
        push(@errorMessages, $error);
    } else {

        $findType = $xmldoc->findnodes('//dataset/identifier');
        if($findType->size() > 0){
        # This is a eml beta6 document
        # Delete the documents mentioned in triples also
        
        $findType = $xmldoc->findnodes('//dataset/triple');
        if($findType->size() > 0){
            foreach $node ($findType->get_nodelist){
            $pushDoc = findValue($node, 'subject');
            
            # If the file is already in the @fileArray then do not add it 
            $alreadyInArray = 0;
            foreach $element (@fileArray){
                if($element eq $pushDoc){
                $alreadyInArray = 1;
                }
            }
            
            if(!$alreadyInArray){
                # If not already in array then delete the file. 
                push (@fileArray, $pushDoc);
                $response = $metacat->delete($pushDoc);
                
                if (! $response) {
                # Could not delete
                #push(@errorMessages, $response);
                push(@errorMessages, $metacat->getMessage());
                push(@errorMessages, "Failed during deleting $pushDoc. Please check if you are authorized to delete this document.\n");
                }
            }
            }
        }
        }
    }
    
    # Delete the main document. 
    if($deleteAll){
        $response = $metacat->delete($docid);  
        if (! $response) {
        # Could not delete
        #push(@errorMessages, $response);
        push(@errorMessages, $metacat->getMessage());
        push(@errorMessages, "Failed during deleting $docid. Please check if you are authorized to delete this document.\n");
        }
    }
    }
    
    if (scalar(@errorMessages)) {
    # If any errors, print them in the response template 
    $$templateVars{'status'} = 'failure';
    $$templateVars{'errorMessages'} = \@errorMessages;
    $error = 1;
    }
    
    # Process the response template
    if($deleteAll){

    $$templateVars{'function'} = "deleted";
    $$templateVars{'section'} = "Deletion Status";
    $template->process( $responseTemplate, $templateVars);
    }
}


################################################################################
# 
# Do data validation and send the data to confirm data template.
#
################################################################################
sub toConfirmData{
    # Check if any invalid parameters
 
    my $invalidParams;
    if (! $error) {
    $invalidParams = validateParameters(0);
    if (scalar(@$invalidParams)) {
        $$templateVars{'status'} = 'failure';
        $$templateVars{'invalidParams'} = $invalidParams;
        $error = 1;
    }
    }

    if (! $error) {
    # If no errors, then print out data in confirm Data template

    $$templateVars{'providerGivenName'} = $FORM::providerGivenName;
    $$templateVars{'providerSurName'} = $FORM::providerSurName;
    if($FORM::site eq "Select your station here."){
        $$templateVars{'site'} = "";
    }else{
        $$templateVars{'site'} = $FORM::site;
    }
    if($FORM::cfg eq "nceas"){
        $$templateVars{'wg'} = \@FORM::wg;
    }
    $$templateVars{'identifier'} = $FORM::identifier;
    $$templateVars{'title'} = $FORM::title;
    $$templateVars{'origNamefirst0'} = $FORM::origNamefirst0;
    $$templateVars{'origNamelast0'} = $FORM::origNamelast0;
    $$templateVars{'origNameOrg'} = $FORM::origNameOrg;
    # $$templateVars{'origRole0'} = $FORM::origRole0;
    $$templateVars{'origDelivery'} = $FORM::origDelivery;
    $$templateVars{'origCity'} = $FORM::origCity;
    if($FORM::origState eq "Select State Here."){
        $$templateVars{'origState'} = "";
    }else{
        $$templateVars{'origState'} = $FORM::origState;
    }
    $$templateVars{'origStateOther'} = $FORM::origStateOther;
    $$templateVars{'origZIP'} = $FORM::origZIP;
    $$templateVars{'origCountry'} = $FORM::origCountry;
    $$templateVars{'origPhone'} = $FORM::origPhone;
    $$templateVars{'origFAX'} = $FORM::origFAX;
    $$templateVars{'origEmail'} = $FORM::origEmail;
    $$templateVars{'useOrigAddress'} = $FORM::useOrigAddress;
    if($FORM::useOrigAddress eq "on"){
        $$templateVars{'origNamefirstContact'} = $FORM::origNamefirst0;
        $$templateVars{'origNamelastContact'} = $FORM::origNamelast0;
        $$templateVars{'origNameOrgContact'} = $FORM::origNameOrg;
        $$templateVars{'origDeliveryContact'} = $FORM::origDelivery; 
        $$templateVars{'origCityContact'} = $FORM::origCity;
        if($FORM::origState eq "Select State Here."){
        $$templateVars{'origStateContact'} = "";
        }else{
        $$templateVars{'origStateContact'} = $FORM::origState;
        }
        $$templateVars{'origStateOtherContact'} = $FORM::origStateOther;
        $$templateVars{'origZIPContact'} = $FORM::origZIP;
        $$templateVars{'origCountryContact'} = $FORM::origCountry;
        $$templateVars{'origPhoneContact'} = $FORM::origPhone;
        $$templateVars{'origFAXContact'} = $FORM::origFAX;
        $$templateVars{'origEmailContact'} = $FORM::origEmail;
    }else{
        $$templateVars{'origNamefirstContact'} = $FORM::origNamefirstContact;
        $$templateVars{'origNamelastContact'} = $FORM::origNamelastContact;
        $$templateVars{'origNameOrgContact'} = $FORM::origNameOrgContact;
        $$templateVars{'origDeliveryContact'} = $FORM::origDeliveryContact; 
        $$templateVars{'origCityContact'} = $FORM::origCityContact;
        if($FORM::origStateContact eq "Select State Here."){
        $$templateVars{'origStateContact'} = "";
        }else{
        $$templateVars{'origStateContact'} = $FORM::origStateContact;
        }
        $$templateVars{'origStateOtherContact'} = $FORM::origStateOtherContact;
        $$templateVars{'origZIPContact'} = $FORM::origZIPContact;
        $$templateVars{'origCountryContact'} = $FORM::origCountryContact;
        $$templateVars{'origPhoneContact'} = $FORM::origPhoneContact;
        $$templateVars{'origFAXContact'} = $FORM::origFAXContact;
        $$templateVars{'origEmailContact'} = $FORM::origEmailContact;    
    }
    $$templateVars{'origNamefirst1'} = $FORM::origNamefirst1;
    $$templateVars{'origNamelast1'} = $FORM::origNamelast1;
    if($FORM::origNamefirst1 eq "" && $FORM::origNamelast1 eq ""){
        $$templateVars{'origRole1'} = "";
    }else{
        $$templateVars{'origRole1'} = $FORM::origRole1;
    }
    $$templateVars{'origNamefirst2'} = $FORM::origNamefirst2;
    $$templateVars{'origNamelast2'} = $FORM::origNamelast2;
    if($FORM::origNamefirst2 eq "" && $FORM::origNamelast2 eq ""){
        $$templateVars{'origRole2'} = "";
    }else{
        $$templateVars{'origRole2'} = $FORM::origRole2;
    }
    $$templateVars{'origNamefirst3'} = $FORM::origNamefirst3;
    $$templateVars{'origNamelast3'} = $FORM::origNamelast3;
    if($FORM::origNamefirst3 eq "" && $FORM::origNamelast3 eq ""){
        $$templateVars{'origRole3'} = "";
    }else{
        $$templateVars{'origRole3'} = $FORM::origRole3;
    }
    $$templateVars{'origNamefirst4'} = $FORM::origNamefirst4;
    $$templateVars{'origNamelast4'} = $FORM::origNamelast4;
    if($FORM::origNamefirst4 eq "" && $FORM::origNamelast4 eq ""){
        $$templateVars{'origRole4'} = "";
    }else{
        $$templateVars{'origRole4'} = $FORM::origRole4;
    }
    $$templateVars{'origNamefirst5'} = $FORM::origNamefirst5;
    $$templateVars{'origNamelast5'} = $FORM::origNamelast5;
    if($FORM::origNamefirst5 eq "" && $FORM::origNamelast5 eq ""){
        $$templateVars{'origRole5'} = "";
    }else{
        $$templateVars{'origRole5'} = $FORM::origRole5;
    }
    $$templateVars{'origNamefirst6'} = $FORM::origNamefirst6;
    $$templateVars{'origNamelast6'} = $FORM::origNamelast6;
    if($FORM::origNamefirst6 eq "" && $FORM::origNamelast6 eq ""){
        $$templateVars{'origRole6'} = "";
    }else{
        $$templateVars{'origRole6'} = $FORM::origRole6;
    }
    $$templateVars{'origNamefirst7'} = $FORM::origNamefirst7;
    $$templateVars{'origNamelast7'} = $FORM::origNamelast7;
    if($FORM::origNamefirst7 eq "" && $FORM::origNamelast7 eq ""){
        $$templateVars{'origRole7'} = "";
    }else{
        $$templateVars{'origRole7'} = $FORM::origRole7;
    }
    $$templateVars{'origNamefirst8'} = $FORM::origNamefirst8;
    $$templateVars{'origNamelast8'} = $FORM::origNamelast8;
    if($FORM::origNamefirst8 eq "" && $FORM::origNamelast8 eq ""){
        $$templateVars{'origRole8'} = "";
    }else{
        $$templateVars{'origRole8'} = $FORM::origRole8;
    }
    $$templateVars{'origNamefirst9'} = $FORM::origNamefirst9;
    $$templateVars{'origNamelast9'} = $FORM::origNamelast9;
    if($FORM::origNamefirst9 eq "" && $FORM::origNamelast9 eq ""){
        $$templateVars{'origRole9'} = "";
    }else{
        $$templateVars{'origRole9'} = $FORM::origRole9;
    }
    $$templateVars{'origNamefirst10'} = $FORM::origNamefirst10;
    $$templateVars{'origNamelast10'} = $FORM::origNamelast10;
    if($FORM::origNamefirst10 eq "" && $FORM::origNamelast10 eq ""){
        $$templateVars{'origRole10'} = "";
    }else{
        $$templateVars{'origRole10'} = $FORM::origRole10;
    }
    $$templateVars{'abstract'} = $FORM::abstract;
    $$templateVars{'keyword0'} = $FORM::keyword0;
    $$templateVars{'kwType0'} = $FORM::kwType0;
    $$templateVars{'kwTh0'} = $FORM::kwTh0;
    $$templateVars{'kwType1'} = $FORM::kwType1;
    $$templateVars{'keyword1'} = $FORM::keyword1;
    $$templateVars{'kwTh1'} = $FORM::kwTh1;
    $$templateVars{'kwType2'} = $FORM::kwType2;
    $$templateVars{'keyword2'} = $FORM::keyword2;
    $$templateVars{'kwTh2'} = $FORM::kwTh2;
    $$templateVars{'kwType3'} = $FORM::kwType3;
    $$templateVars{'keyword3'} = $FORM::keyword3;
    $$templateVars{'kwTh3'} = $FORM::kwTh3;
    $$templateVars{'kwType4'} = $FORM::kwType4;
    $$templateVars{'keyword4'} = $FORM::keyword4;
    $$templateVars{'kwTh4'} = $FORM::kwTh4;
    $$templateVars{'kwType5'} = $FORM::kwType5;
    $$templateVars{'keyword5'} = $FORM::keyword5;
    $$templateVars{'kwTh5'} = $FORM::kwTh5;
    $$templateVars{'kwType6'} = $FORM::kwType6;
    $$templateVars{'keyword6'} = $FORM::keyword6;
    $$templateVars{'kwTh6'} = $FORM::kwTh6;
    $$templateVars{'kwType7'} = $FORM::kwType7;
    $$templateVars{'keyword7'} = $FORM::keyword7;
    $$templateVars{'kwTh7'} = $FORM::kwTh7;
    $$templateVars{'kwType8'} = $FORM::kwType8;
    $$templateVars{'keyword8'} = $FORM::keyword8;
    $$templateVars{'kwTh8'} = $FORM::kwTh8;
    $$templateVars{'kwType9'} = $FORM::kwType9;
    $$templateVars{'keyword9'} = $FORM::keyword9;
    $$templateVars{'kwTh9'} = $FORM::kwTh9;
    $$templateVars{'addComments'} = $FORM::addComments;
    $$templateVars{'useConstraints'} = $FORM::useConstraints;
    $$templateVars{'useConstraintsOther'} = $FORM::useConstraintsOther;
    $$templateVars{'url'} = $FORM::url;
    if($FORM::dataMedium eq "Select type of medium here."){
        $$templateVars{'dataMedium'} = "";
    }else{
        $$templateVars{'dataMedium'} = $FORM::dataMedium;
    }    
    $$templateVars{'dataMediumOther'} = $FORM::dataMediumOther;
    $$templateVars{'beginningYear'} = $FORM::beginningYear;
    $$templateVars{'beginningMonth'} = $FORM::beginningMonth;
    $$templateVars{'beginningDay'} = $FORM::beginningDay;
    $$templateVars{'endingYear'} = $FORM::endingYear;
    $$templateVars{'endingMonth'} = $FORM::endingMonth;
    $$templateVars{'endingDay'} = $FORM::endingDay;
    $$templateVars{'geogdesc'} = $FORM::geogdesc;
    $$templateVars{'useSiteCoord'} = $FORM::useSiteCoord;
    $$templateVars{'latDeg1'} = $FORM::latDeg1;
    $$templateVars{'latMin1'} = $FORM::latMin1;
    $$templateVars{'latSec1'} = $FORM::latSec1;
    $$templateVars{'hemisphLat1'} = $FORM::hemisphLat1;
    $$templateVars{'longDeg1'} = $FORM::longDeg1;
    $$templateVars{'longMin1'} = $FORM::longMin1;
    $$templateVars{'longSec1'} = $FORM::longSec1;
    $$templateVars{'hemisphLong1'} = $FORM::hemisphLong1;
    $$templateVars{'latDeg2'} = $FORM::latDeg2;
    $$templateVars{'latMin2'} = $FORM::latMin2;
    $$templateVars{'latSec2'} = $FORM::latSec2;
    $$templateVars{'hemisphLat2'} = $FORM::hemisphLat2;
    $$templateVars{'longDeg2'} = $FORM::longDeg2;
    $$templateVars{'longMin2'} = $FORM::longMin2;
    $$templateVars{'longSec2'} = $FORM::longSec2;
    $$templateVars{'hemisphLong2'} = $FORM::hemisphLong2;

    $$templateVars{'taxaCount'} = $FORM::taxaCount;
    foreach my $trn (param()) {
        if ($trn =~ /taxonRankName/) {
            my $taxIndex = $trn;
            $taxIndex =~ s/taxonRankName//; # get the index of the parameter 0, ..., 10
            my $trv = "taxonRankValue".$taxIndex;
            if ( $taxIndex =~ /[0-9]/ ){
                if (hasContent(param($trn)) && hasContent(param($trv))) {
                    debug("Registry processing taxon: $trn = ".param($trn)." $trv = ".param($trv));
                    $$templateVars{$trn} = param($trn);
                    $$templateVars{$trv} = param($trv);
                }
            }
        }
    }
    $$templateVars{'taxaAuth'} = $FORM::taxaAuth;

    $$templateVars{'methodTitle'} = $FORM::methodTitle;
    $$templateVars{'methodPara'} = \@FORM::methodPara;
    $$templateVars{'studyExtentDescription'} = $FORM::studyExtentDescription;
    $$templateVars{'samplingDescription'} = $FORM::samplingDescription;
    $$templateVars{'docid'} = $FORM::docid;

    $$templateVars{'section'} = "Confirm Data";
    $template->process( $confirmDataTemplate, $templateVars);

    } else{    
    # Errors from validation function. print the errors out using the response template
    if (scalar(@errorMessages)) {
        $$templateVars{'status'} = 'failure';
        $$templateVars{'errorMessages'} = \@errorMessages;
        $error = 1;
    }
        # Create our HTML response and send it back
    $$templateVars{'function'} = "submitted";
    $$templateVars{'section'} = "Submission Status";
    $template->process( $responseTemplate, $templateVars);
    }
}


################################################################################
# 
# From confirm Data template - user wants to make some changes.
#
################################################################################
sub confirmDataToReEntryData{
    my @sortedSites;
    foreach my $site (sort @sitelist) {
        push(@sortedSites, $site);
    }

    $$templateVars{'siteList'} = \@sortedSites;
    $$templateVars{'section'} = "Re-Entry Form";

    $$templateVars{'providerGivenName'} = $FORM::providerGivenName;
    $$templateVars{'providerSurName'} = $FORM::providerSurName;
    $$templateVars{'site'} = $FORM::site;
    if ($FORM::cfg eq "nceas") {
        my $projects = getProjectList();
        $$templateVars{'projects'} = $projects;
        $$templateVars{'wg'} = \@FORM::wg;
    }
    $$templateVars{'identifier'} = $FORM::identifier;
    $$templateVars{'title'} = $FORM::title;
    $$templateVars{'origNamefirst0'} = $FORM::origNamefirst0;
    $$templateVars{'origNamelast0'} = $FORM::origNamelast0;
    $$templateVars{'origNameOrg'} = $FORM::origNameOrg;
 #   $$templateVars{'origRole0'} = $FORM::origRole0;
    $$templateVars{'origDelivery'} = $FORM::origDelivery;
    $$templateVars{'origCity'} = $FORM::origCity;
    $$templateVars{'origState'} = $FORM::origState;
    $$templateVars{'origStateOther'} = $FORM::origStateOther;
    $$templateVars{'origZIP'} = $FORM::origZIP;
    $$templateVars{'origCountry'} = $FORM::origCountry;
    $$templateVars{'origPhone'} = $FORM::origPhone;
    $$templateVars{'origFAX'} = $FORM::origFAX;
    $$templateVars{'origEmail'} = $FORM::origEmail;
    if ($FORM::useSiteCoord ne "") {
        $$templateVars{'useOrigAddress'} = "CHECKED";
    }else{
        $$templateVars{'useOrigAddress'} = $FORM::useOrigAddress;
    }
    $$templateVars{'origNamefirstContact'} = $FORM::origNamefirstContact;
    $$templateVars{'origNamelastContact'} = $FORM::origNamelastContact;
    $$templateVars{'origNameOrgContact'} = $FORM::origNameOrgContact;
    $$templateVars{'origDeliveryContact'} = $FORM::origDeliveryContact; 
    $$templateVars{'origCityContact'} = $FORM::origCityContact;
    $$templateVars{'origStateContact'} = $FORM::origStateContact;
    $$templateVars{'origStateOtherContact'} = $FORM::origStateOtherContact;
    $$templateVars{'origZIPContact'} = $FORM::origZIPContact;
    $$templateVars{'origCountryContact'} = $FORM::origCountryContact;
    $$templateVars{'origPhoneContact'} = $FORM::origPhoneContact;
    $$templateVars{'origFAXContact'} = $FORM::origFAXContact;
    $$templateVars{'origEmailContact'} = $FORM::origEmailContact;    
    $$templateVars{'origNamefirst1'} = $FORM::origNamefirst1;
    $$templateVars{'origNamelast1'} = $FORM::origNamelast1;
    $$templateVars{'origRole1'} = $FORM::origRole1;
    $$templateVars{'origNamefirst2'} = $FORM::origNamefirst2;
    $$templateVars{'origNamelast2'} = $FORM::origNamelast2;
    $$templateVars{'origRole2'} = $FORM::origRole2;
    $$templateVars{'origNamefirst3'} = $FORM::origNamefirst3;
    $$templateVars{'origNamelast3'} = $FORM::origNamelast3;
    $$templateVars{'origRole3'} = $FORM::origRole3;
    $$templateVars{'origNamefirst4'} = $FORM::origNamefirst4;
    $$templateVars{'origNamelast4'} = $FORM::origNamelast4;
    $$templateVars{'origRole4'} = $FORM::origRole4;
    $$templateVars{'origNamefirst5'} = $FORM::origNamefirst5;
    $$templateVars{'origNamelast5'} = $FORM::origNamelast5;
    $$templateVars{'origRole5'} = $FORM::origRole5;
    $$templateVars{'origNamefirst6'} = $FORM::origNamefirst6;
    $$templateVars{'origNamelast6'} = $FORM::origNamelast6;
    $$templateVars{'origRole6'} = $FORM::origRole6;
    $$templateVars{'origNamefirst7'} = $FORM::origNamefirst7;
    $$templateVars{'origNamelast7'} = $FORM::origNamelast7;
    $$templateVars{'origRole7'} = $FORM::origRole7;
    $$templateVars{'origNamefirst8'} = $FORM::origNamefirst8;
    $$templateVars{'origNamelast8'} = $FORM::origNamelast8;
    $$templateVars{'origRole8'} = $FORM::origRole8;
    $$templateVars{'origNamefirst9'} = $FORM::origNamefirst9;
    $$templateVars{'origNamelast9'} = $FORM::origNamelast9;
    $$templateVars{'origRole9'} = $FORM::origRole9;
    $$templateVars{'origNamefirst10'} = $FORM::origNamefirst10;
    $$templateVars{'origNamelast10'} = $FORM::origNamelast10;
    $$templateVars{'origRole10'} = $FORM::origRole10;
    $$templateVars{'abstract'} = $FORM::abstract;
    $$templateVars{'keyword0'} = $FORM::keyword0;
    $$templateVars{'kwType0'} = $FORM::kwType0;
    $$templateVars{'kwTh0'} = $FORM::kwTh0;
    $$templateVars{'kwType1'} = $FORM::kwType1;
    $$templateVars{'keyword1'} = $FORM::keyword1;
    $$templateVars{'kwTh1'} = $FORM::kwTh1;
    $$templateVars{'kwType2'} = $FORM::kwType2;
    $$templateVars{'keyword2'} = $FORM::keyword2;
    $$templateVars{'kwTh2'} = $FORM::kwTh2;
    $$templateVars{'kwType3'} = $FORM::kwType3;
    $$templateVars{'keyword3'} = $FORM::keyword3;
    $$templateVars{'kwTh3'} = $FORM::kwTh3;
    $$templateVars{'kwType4'} = $FORM::kwType4;
    $$templateVars{'keyword4'} = $FORM::keyword4;
    $$templateVars{'kwTh4'} = $FORM::kwTh4;
    $$templateVars{'kwType5'} = $FORM::kwType5;
    $$templateVars{'keyword5'} = $FORM::keyword5;
    $$templateVars{'kwTh5'} = $FORM::kwTh5;
    $$templateVars{'kwType6'} = $FORM::kwType6;
    $$templateVars{'keyword6'} = $FORM::keyword6;
    $$templateVars{'kwTh6'} = $FORM::kwTh6;
    $$templateVars{'kwType7'} = $FORM::kwType7;
    $$templateVars{'keyword7'} = $FORM::keyword7;
    $$templateVars{'kwTh7'} = $FORM::kwTh7;
    $$templateVars{'kwType8'} = $FORM::kwType8;
    $$templateVars{'keyword8'} = $FORM::keyword8;
    $$templateVars{'kwTh8'} = $FORM::kwTh8;
    $$templateVars{'kwType9'} = $FORM::kwType9;
    $$templateVars{'keyword9'} = $FORM::keyword9;
    $$templateVars{'kwTh9'} = $FORM::kwTh9;
    $$templateVars{'addComments'} = $FORM::addComments;
    $$templateVars{'useConstraints'} = $FORM::useConstraints;
    $$templateVars{'useConstraintsOther'} = $FORM::useConstraintsOther;
    $$templateVars{'url'} = $FORM::url;
    $$templateVars{'dataMedium'} = $FORM::dataMedium;
    $$templateVars{'dataMediumOther'} = $FORM::dataMediumOther;
    $$templateVars{'beginningYear'} = $FORM::beginningYear;
    $$templateVars{'beginningMonth'} = $FORM::beginningMonth;
    $$templateVars{'beginningDay'} = $FORM::beginningDay;
    $$templateVars{'endingYear'} = $FORM::endingYear;
    $$templateVars{'endingMonth'} = $FORM::endingMonth;
    $$templateVars{'endingDay'} = $FORM::endingDay;
    $$templateVars{'geogdesc'} = $FORM::geogdesc;
    if($FORM::useSiteCoord ne ""){
    $$templateVars{'useSiteCoord'} = "CHECKED";
    }else{
    $$templateVars{'useSiteCoord'} = "";
    }
    $$templateVars{'latDeg1'} = $FORM::latDeg1;
    $$templateVars{'latMin1'} = $FORM::latMin1;
    $$templateVars{'latSec1'} = $FORM::latSec1;
    $$templateVars{'hemisphLat1'} = $FORM::hemisphLat1;
    $$templateVars{'longDeg1'} = $FORM::longDeg1;
    $$templateVars{'longMin1'} = $FORM::longMin1;
    $$templateVars{'longSec1'} = $FORM::longSec1;
    $$templateVars{'hemisphLong1'} = $FORM::hemisphLong1;
    $$templateVars{'latDeg2'} = $FORM::latDeg2;
    $$templateVars{'latMin2'} = $FORM::latMin2;
    $$templateVars{'latSec2'} = $FORM::latSec2;
    $$templateVars{'hemisphLat2'} = $FORM::hemisphLat2;
    $$templateVars{'longDeg2'} = $FORM::longDeg2;
    $$templateVars{'longMin2'} = $FORM::longMin2;
    $$templateVars{'longSec2'} = $FORM::longSec2;
    $$templateVars{'hemisphLong2'} = $FORM::hemisphLong2;
    $$templateVars{'taxaCount'} = $FORM::taxaCount;
    foreach my $trn (param()) {
        if ($trn =~ /taxonRankName/) {
            my $taxIndex = $trn;
            $taxIndex =~ s/taxonRankName//; # get the index of the parameter 0, ..., 10
            my $trv = "taxonRankValue".$taxIndex;
            if ( $taxIndex =~ /[0-9]/ ){
                if (hasContent(param($trn)) && hasContent(param($trv))) {
                    debug("Registry processing taxon: $trn = ".param($trn)." $trv = ".param($trv));
                    $$templateVars{$trn} = param($trn);
                    $$templateVars{$trv} = param($trv);
                }
            }
        }
    }
    $$templateVars{'taxaAuth'} = $FORM::taxaAuth;
    $$templateVars{'methodTitle'} = $FORM::methodTitle;
    $$templateVars{'methodPara'} = \@FORM::methodPara;
    $$templateVars{'studyExtentDescription'} = $FORM::studyExtentDescription;
    $$templateVars{'samplingDescription'} = $FORM::samplingDescription;
    $$templateVars{'docid'} = $FORM::docid;

    $$templateVars{'form'} = 're_entry';
    $template->process( $entryFormTemplate, $templateVars);
}


################################################################################
# 
# check if there is multiple occurence of the given tag and find its value.
#
################################################################################

sub findValue {
    my $node = shift;
    my $value = shift;
    my $result;
    my $tempNode;

    $result = $node->findnodes("./$value");
    if ($result->size > 1) {
        errMoreThanOne("$value");
    } else {
        foreach $tempNode ($result->get_nodelist){
            #print $tempNode->nodeName().":".$tempNode->textContent();
            #print "\n";
            return $tempNode->textContent();
        }
    }
}


################################################################################
# 
# check if given tags has any children. if not return the value
#
################################################################################
sub findValueNoChild {
    my $node = shift;
    my $value = shift;
    my $tempNode;
    my $childNodes;
    my $result;
    my $error;

    $result = $node->findnodes("./$value");
    if($result->size > 1){
       errMoreThanOne("$value");
    } else {
        foreach $tempNode ($result->get_nodelist) {
            $childNodes = $tempNode->childNodes;
            if ($childNodes->size() > 1) {
                $error ="The tag $value has children which cannot be shown using the form. Please use Morpho to edit this document";    
                push(@errorMessages, $error);
                #if ($DEBUG == 1){ print $error."\n";}
            } else {
                #print $tempNode->nodeName().":".$tempNode->textContent();
                #print "\n";
                return $tempNode->textContent();
            }
        }
    }
}


################################################################################
# 
# check if given tags are children of given node.
#
################################################################################
sub dontOccur {
    my $node = shift;
    my $value = shift;
    my $errVal = shift;

    my $result = $node->findnodes("$value");
    if($result->size > 0){
        $error ="One of the following tags found: $errVal. Please use Morpho to edit this document";
        push(@errorMessages, $error."\n");
        #if ($DEBUG == 1){ print $error;}
    } 
}


################################################################################
# 
# print out error for more than one occurence of a given tag
#
################################################################################
sub errMoreThanOne {
    my $value = shift;
    my $error ="More than one occurence of the tag $value found. Please use Morpho to edit this document";
    push(@errorMessages, $error."\n");
    # if ($DEBUG == 1){ print $error;}
}


################################################################################
# 
# print out error for more than given number of occurences of a given tag
#
################################################################################
sub errMoreThanN {
    my $value = shift;
    my $error ="More occurences of the tag $value found than that can be shown in the form. Please use Morpho to edit this document";
    push(@errorMessages, $error);
    #if ($DEBUG == 1){ print $error."\n";}
}


################################################################################
# 
# convert coord to degrees, minutes and seconds form. 
#
################################################################################
#sub convertCoord {
#    my $wx = shift;
#    print $deg." ".$min." ".$sec;
#    print "\n";
#}


################################################################################
# 
# print debugging messages to stderr
#
################################################################################
sub debug {
    my $msg = shift;
    
    if ($debug) {
        print STDERR "$msg\n";
    }
}

################################################################################
# 
# get the list of projects
#
################################################################################
sub getProjectList {
    
    use NCEAS::AdminDB;
    my $admindb = NCEAS::AdminDB->new();
    $admindb->connect($nceas_db, $nceas_db_user, $nceas_db_password);
    my $projects = $admindb->getProjects();
    #my $projects = getTestProjectList();
    return $projects;
}

################################################################################
# 
# get a test list of projects for use only in testing where the NCEAS
# admin db is not available.
#
################################################################################
sub getTestProjectList {
    # This block is for testing only!  Remove for production use
    my @row1;
    $row1[0] = 6000; $row1[1] = 'Andelman'; $row1[2] = 'Sandy'; $row1[3] = 'The very long and windy path to an apparent ecological conclusion: statistics lie';
    my @row2; 
    $row2[0] = 7000; $row2[1] = 'Bascompte'; $row2[2] = 'Jordi'; $row2[3] = 'Postdoctoral Fellow';
    my @row3; 
    $row3[0] = 7001; $row3[1] = 'Hackett'; $row3[2] = 'Edward'; $row3[3] = 'Sociology rules the world';
    my @row4; 
    $row4[0] = 7002; $row4[1] = 'Jones'; $row4[2] = 'Matthew'; $row4[3] = 'Informatics rules the world';
    my @row5; 
    $row5[0] = 7003; $row5[1] = 'Schildhauer'; $row5[2] = 'Mark'; $row5[3] = 'Excel rocks my world, assuming a, b, and c';
    my @row6; 
    $row6[0] = 7004; $row6[1] = 'Rogers'; $row6[2] = 'Bill'; $row6[3] = 'Graduate Intern';
    my @row7; 
    $row7[0] = 7005; $row7[1] = 'Zedfried'; $row7[2] = 'Karl'; $row7[3] = 'A multivariate analysis of thing that go bump in the night';
    my @projects;
    $projects[0] = \@row1;
    $projects[1] = \@row2;
    $projects[2] = \@row3;
    $projects[3] = \@row4;
    $projects[4] = \@row5;
    $projects[5] = \@row6;
    $projects[6] = \@row7;
    return \@projects;
}
